package pps.core.system.service;

import pps.cloud.sms.service.SmsService;
import pps.cloud.sms.service.data.MessageParam;
import pps.core.common.utils.PatternUtil;
import pps.core.common.utils.VerificationCode;
import pps.core.system.cache.LoginVerCodeCache;
import pps.core.system.error.LoginError;
import pps.core.system.service.data.sys_login.SendMessageInput;
import pps.core.system.service.data.sys_login.SmsCodeOutput;
import pps.core.system.service.data.sys_login.VerificationCodeInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.Map;

/**
 * @author lixueyan
 * @date 2023/4/17 0017 10:19
 */
@XService
public class SendMessageService {

    @XText("验证码")
    @XApiPost(anonymous = true)
    public XSingleResult<SmsCodeOutput> getSmsCode(XContext context , VerificationCodeInput input ) {
        Map<String, Object> codeMap = VerificationCode.generateCodeAndPic();
        // 将四位数字的验证码保存到缓存中。
        LoginVerCodeCache cache = new LoginVerCodeCache();
        cache.setCodeKey(input.getCodeIden());
        cache.setCode(codeMap.get("code").toString());
        LoginVerCodeCache.set(context,cache);

        SmsCodeOutput output = new SmsCodeOutput();
        output.setSmsCode(cache.getCode());
        return XSingleResult.success(output);
    }

    @XText("登陆发送短信")
    @XApiPost(anonymous = true)
    public XServiceResult sendMessage(XContext context, SendMessageInput input) {
        checkCode(context,input.getCode(),input.getCodeIden());
        String[] split = input.getPhoneStr().split(",");
        for (String s : split) {
            if(!PatternUtil.checkPhone(s)){
                return XServiceResult.error(context,LoginError.NotExistPhone);
            }
        }

        String smsContent = input.getSmsContent();
        SmsService smsService = context.getBean(SmsService.class);
        MessageParam param = new MessageParam();
        param.setPhoneNum(input.getPhoneStr());
        param.setVariable(smsContent);

        XServiceResult result = smsService.sendMessage(context, param);
        if(!result.isSuccess()){
            XServiceResult.error(context,LoginError.ExceptionSmsCode);
        }

        return XServiceResult.OK;
    }

    private void checkCode(XContext context,String code,String codeIden){
        if(code == null) {
            throw new XServiceException(context,LoginError.NotExistCode);
        }
        LoginVerCodeCache isUse = LoginVerCodeCache.exist(context , codeIden);
        if(isUse == null) {
            throw new XServiceException(context,LoginError.NotUsedCode);
        }
        if(!isUse.getCode().toLowerCase().equals(code.toLowerCase())) {
            throw new XServiceException(context,LoginError.NotCompareCode);
        }
    }
}
