package pps.core.system.service;

import cn.hutool.core.lang.TypeReference;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.baseframe.service.FormFuncPrecedureCloudService;
import pps.cloud.baseframe.service.FormInputRecordCloudService;
import pps.cloud.baseframe.service.FormWorkflowRelCloudService;
import pps.cloud.baseframe.service.data.form_func_precedure.FormFuncPrecedureInput;
import pps.cloud.baseframe.service.data.form_func_precedure.FormFuncPrecedureOutput;
import pps.cloud.baseframe.service.data.form_input_record.GetFormInputRecordInput;
import pps.cloud.baseframe.service.data.form_input_record.GetFormInputRecordOutput;
import pps.cloud.baseframe.service.data.form_workflow_rel.GetFormWorkflowRelInput;
import pps.cloud.system.service.SysInformOperateTemplateCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.common.config.MqConfig;
import pps.core.common.mq.SysInformSendMqMessage;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.DynObj;
import pps.core.common.utils.TraceHelper;
import pps.core.system.constant.SysInformReceiveTypeConstant;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_inform.SendSysInformInput;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

@XService
public class SysInformOperateTemplateCloudServiceImpl implements SysInformOperateTemplateCloudService {

    @Override
    public XServiceResult sendSysInform(XContext context, SendSysInformInfoInput input) {
        TraceHelper.buildTraceBuilder(context);
        try {

            TraceHelper.appendTraceLog(context, "接收到流程操作，进入到通知，参数为：", input.toString());
            String sysInformId = UUID.randomUUID().toString();
            XTransactionHelper.begin(context, () -> {

                SysInformOperateTemplateMapper mapper = context.getBean(SysInformOperateTemplateMapper.class);
                QueryWrapper<SysInformOperateTemplateEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysInformOperateTemplateEnt::getFormWorkflowRelId,input.getFormWorkflowRelId())
                        .eq(SysInformOperateTemplateEnt::getOperateType,input.getOperateType());
                SysInformOperateTemplateEnt templateEnt = mapper.selectOne(wrapper);
                if (templateEnt == null){
                    TraceHelper.appendTraceLog(context, "ERROR", "此流程未配置该操作状态!");
                    return XServiceResult.error(200,"此流程未配置该操作状态!");
                }

                SysInformTemplateReceiveTypeMapper typeMapper = context.getBean(SysInformTemplateReceiveTypeMapper.class);
                QueryWrapper<SysInformTemplateReceiveTypeEnt> typeWrapper = new QueryWrapper<>();
                typeWrapper.lambda().eq(SysInformTemplateReceiveTypeEnt::getInformOperateTemplateId,templateEnt.getId());
                List<SysInformTemplateReceiveTypeEnt> typeEnts = typeMapper.selectList(typeWrapper);


                SysInformTemplateReceiveTypeDetailMapper typeDetailMapper = context.getBean(SysInformTemplateReceiveTypeDetailMapper.class);
                QueryWrapper<SysInformTemplateReceiveTypeDetailEnt> typeDetailWrapper = new QueryWrapper<>();
                typeDetailWrapper.lambda().eq(SysInformTemplateReceiveTypeDetailEnt::getInformOperateTemplateId,templateEnt.getId());
                List<SysInformTemplateReceiveTypeDetailEnt> typeDetailEnts = typeDetailMapper.selectList(typeDetailWrapper);

                FormInputRecordCloudService formInputRecord = context.getBean(FormInputRecordCloudService.class);
                GetFormInputRecordInput recordInput = new GetFormInputRecordInput();
                recordInput.setId(input.getFormInputRecordId());
                XSingleResult<GetFormInputRecordOutput> inputRecord = formInputRecord.getFormInputRecord(context, recordInput);
                GetFormInputRecordOutput inputRecordResult = inputRecord.getResult();
                if (inputRecordResult == null){
                    TraceHelper.appendTraceLog(context, "ERROR", "未查到表单填报记录!");
                    return XServiceResult.error(200,"未查到表单填报记录!");
                }

                if (XStringUtils.isNotEmpty(templateEnt.getTitle())){
                    String title = templateEnt.getTitle();
                    if (title.contains("$formname")){
                        title = title.replace("$formname",inputRecordResult.getFormInputName());
                    }
                    templateEnt.setTitle(title);
                    TraceHelper.appendTraceLog(context, "发送通知的标题为：", title);
                }
                if (XStringUtils.isEmpty(input.getWorkflowRemark())){
                    input.setWorkflowRemark(" ");
                }
                if (XStringUtils.isNotEmpty(templateEnt.getMsg())){
                    String msg = templateEnt.getMsg();
                    if (msg.contains("$formname")){
                        msg = msg.replace("$formname",inputRecordResult.getFormInputName());

                    }
                    if (msg.contains("$requestreason")){

                        msg = msg.replace("$requestreason",input.getWorkflowRemark());
                    }
                    if (msg.contains("$reviewnote")){
                        msg = msg.replace("$reviewnote",input.getWorkflowRemark());
                    }
                    templateEnt.setMsg(msg);
                    TraceHelper.appendTraceLog(context, "发送通知的内容为：", msg);
                }
                Date date = new Date();
                SysInformEnt sysInformEnt = new SysInformEnt();
                sysInformEnt.setId(sysInformId);
                sysInformEnt.setMobileMsg(templateEnt.getMobileMsg());
                sysInformEnt.setSystemMsg(templateEnt.getSystemMsg());
                sysInformEnt.setTitle(templateEnt.getTitle());
                sysInformEnt.setMsg(templateEnt.getMsg());
                sysInformEnt.setSendType(0);
                sysInformEnt.setStatus(0);
                sysInformEnt.setCreateTime(date);
                if(null != templateEnt.getEffectiveDays())
                    sysInformEnt.setEffectiveDays(templateEnt.getEffectiveDays());
                if (XStringUtils.isNotEmpty(input.getFormWorkflowRelId())){
                    FormWorkflowRelCloudService bean = context.getBean(FormWorkflowRelCloudService.class);
                    GetFormWorkflowRelInput relInput = new GetFormWorkflowRelInput();
                    relInput.setId(input.getFormWorkflowRelId());
                    XSingleResult<GetFormWorkflowRelInput> formWorkflowRelById = bean.getFormWorkflowRelById(context, relInput);
                    GetFormWorkflowRelInput rel = formWorkflowRelById.getResult();
                    if (rel != null){
                        sysInformEnt.setOuId(rel.getOuId());
                        if (XStringUtils.isNotEmpty(rel.getOuId())){
                            SysOrganizationMapper ouMapper = context.getBean(SysOrganizationMapper.class);
                            QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
                            queryWrapper.lambda().eq(SysOrganizationEnt::getId,rel.getOuId()).eq(SysOrganizationEnt::getIsDeleted, SysOrganizationCode.NORMAL);
                            SysOrganizationEnt organizationEnt = ouMapper.selectOne(queryWrapper);
                            if (organizationEnt != null){
                                sysInformEnt.setOuName(organizationEnt.getOuName());
                            }

                        }

                    }

                }



                List<SysInformReceiveTypeEnt> typeList = new ArrayList<>();
                List<SysInformReceiveTypeDetailEnt> typeDetailList = new ArrayList<>();
                SysInformMapper informMapper = context.getBean(SysInformMapper.class);
                SysInformReceiveTypeMapper receiveTypeMapper = context.getBean(SysInformReceiveTypeMapper.class);
                SysInformReceiveTypeDetailMapper receiveTypeDetailMapper = context.getBean(SysInformReceiveTypeDetailMapper.class);

                for (SysInformTemplateReceiveTypeEnt typeEnt : typeEnts) {
                    if (typeEnt.getRecieveType() != null){
                        SysInformReceiveTypeEnt receiveTypeEnt = new SysInformReceiveTypeEnt();
                        String typeEntId = UUID.randomUUID().toString();
                        receiveTypeEnt.setId(typeEntId);
                        receiveTypeEnt.setInformId(sysInformId);
                        receiveTypeEnt.setRecieveType(typeEnt.getRecieveType());
                        receiveTypeEnt.setRecieveTypeName(typeEnt.getRecieveTypeName());
                        typeList.add(receiveTypeEnt);
                        switch (typeEnt.getRecieveType()){
                            case 1://人员
                                TraceHelper.appendTraceLog(context, "接收类型为人员", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> user = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : user) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(detailEnt.getOuId());
                                    typeDetailEnt.setOuName(detailEnt.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为人员的接收信息", typeDetailList);
                                break;
                            case 2://收件组
                                TraceHelper.appendTraceLog(context, "接收类型为收件组", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> group = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : group) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(detailEnt.getOuId());
                                    typeDetailEnt.setOuName(detailEnt.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为收件组的接收信息", typeDetailList);
                                break;
                            case 3://角色
                                TraceHelper.appendTraceLog(context, "接收类型为角色", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> roles = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : roles) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(detailEnt.getOuId());
                                    typeDetailEnt.setOuName(detailEnt.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为角色的接收信息", typeDetailList);
                                break;
                            case 4://机构
                                TraceHelper.appendTraceLog(context, "接收类型为机构", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> ous = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : ous) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(detailEnt.getOuId());
                                    typeDetailEnt.setOuName(detailEnt.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为机构的接收信息", typeDetailList);
                                break;
                            case 5://角色-提交人本级单位
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人本级单位", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> thisLevel = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                SysUserOrganizationRelViewMapper bean = context.getBean(SysUserOrganizationRelViewMapper.class);
                                List<SysUserOrganizationRelView> orgListByUserId = bean.getOrgListByUserId(inputRecordResult.getCreateUserid());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : thisLevel) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(orgListByUserId.get(0).getOuId());
                                    typeDetailEnt.setOuName(orgListByUserId.get(0).getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人本级单位的接收信息", typeDetailList);
                                break;
                            case 6://角色-提交人二级单位
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人二级单位", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> twoLevel = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                SysOrganizationView twoView = new SysOrganizationView();
                                twoView.setOuLevel("ADMINISTRATIVE");
                                twoView.setUserId(inputRecordResult.getCreateUserid());
                                GetSysOrganizationViewOutput twoOrganization = getSysOrganizationByOuLevelAndUserId(context, twoView);
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : twoLevel) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(twoOrganization.getId());
                                    typeDetailEnt.setOuName(twoOrganization.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人二级单位的接收信息", typeDetailList);
                                break;
                            case 7://角色-提交人所属企业
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人所属企业", "开始拼接参数");
                                List<SysInformTemplateReceiveTypeDetailEnt> affiliated = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt -> typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                SysOrganizationView affiliatedView = new SysOrganizationView();
                                affiliatedView.setOuLevel("REGION_FIRM");
                                affiliatedView.setUserId(inputRecordResult.getCreateUserid());
                                GetSysOrganizationViewOutput affiliatedOrganization = getSysOrganizationByOuLevelAndUserId(context, affiliatedView);
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : affiliated) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(affiliatedOrganization.getId());
                                    typeDetailEnt.setOuName(affiliatedOrganization.getOuName());
                                    typeDetailEnt.setObjId(detailEnt.getObjId());
                                    typeDetailEnt.setObjName(detailEnt.getObjName());
                                    typeDetailList.add(typeDetailEnt);
                                }
                                TraceHelper.appendTraceLog(context, "接收类型为角色-提交人所属企业的接收信息", typeDetailList);
                                break;
                            case 8://存储过程
                                TraceHelper.appendTraceLog(context, "接收类型为存储过程", "开始拼接参数");
                                FormFuncPrecedureCloudService precedureCloudService = context.getBean(FormFuncPrecedureCloudService.class);
                                SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
                                CreateSysDictionaryInput viewInput = new CreateSysDictionaryInput();
                                viewInput.setAlias("APP_SERVICE_CONF");
                                XListResult<QuerySysDictionaryViewOutput> dictionarys = dictService.querySysDictionarysByAlias(context, viewInput);
                                List<SysInformTemplateReceiveTypeDetailEnt> function = typeDetailEnts.stream().filter(sysInformTemplateReceiveTypeDetailEnt ->
                                        typeEnt.getId().equals(sysInformTemplateReceiveTypeDetailEnt.getInformTemplateRecieveTypeId())).collect(Collectors.toList());
                                for (SysInformTemplateReceiveTypeDetailEnt detailEnt : function) {
                                    SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                                    typeDetailEnt.setId(UUID.randomUUID().toString());
                                    typeDetailEnt.setInformId(sysInformId);
                                    typeDetailEnt.setInformRecieveTypeId(typeEntId);
                                    typeDetailEnt.setOuId(detailEnt.getOuId());
                                    typeDetailEnt.setOuName(detailEnt.getOuName());
                                    //存储过程名称
                                    String objId = detailEnt.getObjId();
                                    FormFuncPrecedureInput precedureInput = new FormFuncPrecedureInput();
                                    precedureInput.setPrecedureName(objId);
                                    XSingleResult<FormFuncPrecedureOutput> precedureName = precedureCloudService.getFormFuncPrecedureByPrecedureName(context, precedureInput);
                                    FormFuncPrecedureOutput result = precedureName.getResult();

                                    StringBuilder builder = new StringBuilder();
                                    builder.append(objId);
                                    TraceHelper.appendTraceLog(context, "存储过程名称", objId);

                                    builder.append(" (");
                                    if (result != null && XStringUtils.isNotEmpty(result.getInputParams())){
                                        String[] params = result.getInputParams().split(",");
                                        TraceHelper.appendTraceLog(context, "存储过程参数为", Arrays.toString(params));
                                        for (String param : params) {
                                            if (XStringUtils.isNotEmpty(inputRecordResult.getFilterData())){
                                                Map<String,String> filterData = JSON.parseObject(inputRecordResult.getFilterData(), new TypeReference<HashMap<String, String>>() {
                                                });
                                                String value = filterData.get(param);
                                                if (XStringUtils.isEmpty(value)){
                                                    //容错
                                                    value=filterData.get(param+"Id");
                                                }
                                                if (value==null){
                                                    value="";
                                                }
                                                builder.append(String.format(" '%s' ", value));
                                                TraceHelper.appendTraceLog(context, "存储过程参数值", value);
                                                builder.append(",");
                                            }
                                        }
                                        builder.deleteCharAt(builder.length() - 1);
                                    }
                                    builder.append(" )");

                                    if (dictionarys.getResult() != null && result != null) {
                                        List<QuerySysDictionaryViewOutput> items = dictionarys.getResult();
                                        for (QuerySysDictionaryViewOutput item : items) {
                                            if (result.getDataSource().equals(item.getDicKey())) {
                                                typeDetailEnt.setObjId("call "+item.getShortName()+"."+builder);
                                                TraceHelper.appendTraceLog(context, "完整的存储过程", "call "+item.getShortName()+"."+builder);
                                                typeDetailEnt.setObjName(detailEnt.getObjName());
                                                break;
                                            }
                                        }
                                    }

                                    typeDetailList.add(typeDetailEnt);
                                }
                                break;
                        }
                    }
                }

                informMapper.insert(sysInformEnt);

                if (!CollectionUtils.isEmpty(typeList)){
                    typeList.forEach(receiveTypeMapper::insert);
                }

                if (!CollectionUtils.isEmpty(typeDetailList)){
                    typeDetailList.forEach(receiveTypeDetailMapper::insert);
                }
                return XServiceResult.OK;
            });
            //调用发送通知
            SendSysInformInput informInput = new SendSysInformInput();
            informInput.setId(sysInformId);
            sendSysInformMq(context,informInput);
            return XServiceResult.OK;
        }catch (Exception e){
            TraceHelper.appendTraceLog(context, e);
            return XServiceResult.error(e);
        }finally {
            TraceLogger.INSTANCE.add(context);
        }

    }

    private XServiceResult sendSysInformMq(XContext context, SendSysInformInput input){
        return XTransactionHelper.begin(context , ()->{
            //判断创建和更新操作
            Date date = new Date();

            SysInformMapper mapper = context.getBean(SysInformMapper.class);
            QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysInformEnt::getId , input.getId());
            SysInformEnt ent = mapper.selectOne(wrapper);
            if(ent == null)
                return XServiceResult.error(context, XError.NotFound);
            if(ent.getStatus() >= 1)
                return XServiceResult.error(200, "数据状态不支持发送！");
            SysInformSendMqMessage sysInformSendMqMessage =new SysInformSendMqMessage();
            sysInformSendMqMessage.setId(input.getId());
            sendMqSysInform(context, sysInformSendMqMessage);
            //执行完成后，更新状态
            ent.setStatus(1);
            ent.setModifyTime(date);
            ent.setSendTime(date);
            mapper.updateById(ent);
            return XServiceResult.OK;
        });
    }

    public GetSysOrganizationViewOutput getSysOrganizationByOuLevelAndUserId(XContext context, SysOrganizationView view) {
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        view.setEndTime(new Date());
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndUserId(view);
        return XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
    }

    public void sendMqSysInform(XContext context, SysInformSendMqMessage sysInformSendMqMessage) {
        String schedulTopic = MqConfig.sysInformSendMq.getValue(context);
        XMqProduceQueue queue = new XMqProduceQueue.Builder().queue(schedulTopic).build();
        XMqProducer<SysInformSendMqMessage> producer = XMqFactory.createProducer(context, queue, SysInformSendMqMessage.class);
        producer.publish(context, sysInformSendMqMessage).logIfFail(context);
    }
}
