package pps.core.system.service;

import java.util.ArrayList;
import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.baseframe.service.FormWorkflowRelCloudService;
import pps.cloud.baseframe.service.data.form_workflow_rel.GetFormWorkflowRelInput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.common.config.MqConfig;
import pps.core.common.mq.PushDispLngShippingScheduleDateValueMessage;
import pps.core.common.mq.SysInformSendMqMessage;
import pps.core.common.session.PpsUserSession;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_inform_operate_template.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import pps.core.system.service.data.sys_inform_template_receive_type.CreateSysInformTemplateReceiveTypeInput;
import pps.core.system.service.data.sys_inform_template_receive_type.GetSysInformTemplateReceiveTypeOutput;
import pps.core.system.service.data.sys_inform_template_receive_type_detail.CreateSysInformTemplateReceiveTypeDetailInput;
import pps.core.system.service.data.sys_inform_template_receive_type_detail.GetSysInformTemplateReceiveTypeDetailOutput;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.base.util.XStringUtils;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import static net.sf.jsqlparser.util.validation.metadata.NamedObject.view;

@XService
@XText("通知配置管理")
public class SysInformOperateTemplateService {



    @XText("根据流程id获取所有通知模版")
    @XApiPost
    public XPageResult<QuerySysInformOperateTemplateViewOutput> querySysInformOperateTemplateView(XContext context, QuerySysInformOperateTemplateViewInput input){

        SysInformOperateTemplateViewMapper mapper = context.getBean(SysInformOperateTemplateViewMapper.class);
        SysInformOperateTemplateView record = new SysInformOperateTemplateView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysInformOperateTemplateView> list = mapper.selectList(record);
        PageInfo<SysInformOperateTemplateView> pageInfo = new PageInfo<>(list);
        List<QuerySysInformOperateTemplateViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysInformOperateTemplateViewOutput.class);
        //操作状态
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        CreateSysDictionaryInput unitLevelInput = new CreateSysDictionaryInput();
        unitLevelInput.setAlias("OPERATE_TEMPLATE");
        XListResult<QuerySysDictionaryViewOutput> operateTemplates = dictService.querySysDictionarysByAlias(context, unitLevelInput);
        for (QuerySysInformOperateTemplateViewOutput output : outputs) {
            if (XStringUtils.isNotEmpty(output.getOperateType())){
                for (QuerySysDictionaryViewOutput dict : operateTemplates.getResult()) {
                    if (dict.getDicKey().equals(output.getOperateType())){
                        output.setOperateTypeName(dict.getDicName());
                    }
                }
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());

    }

    @XText("新建通知配置")
    @XApiPost
    public XServiceResult createSysInformOperateTemplate(XContext context, CreateSysInformOperateTemplateInput input)throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysInformOperateTemplateMapper mapper = context.getBean(SysInformOperateTemplateMapper.class);
            List<String> workflowRelIds = input.getFormWorkflowRelIds();

            QueryWrapper<SysInformOperateTemplateEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().in(SysInformOperateTemplateEnt::getFormWorkflowRelId,workflowRelIds)
                    .eq(SysInformOperateTemplateEnt::getOperateType,input.getOperateType());
            List<SysInformOperateTemplateEnt> templateEnts = mapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(templateEnts)){
               return XServiceResult.error(200,"流程ID为:"+templateEnts.get(0).getFormWorkflowRelId()+"已配置了"+templateEnts.get(0).getOperateType()+"!");
            }
            List<SysInformTemplateReceiveTypeEnt> typeEnts = new ArrayList<>();
            List<SysInformTemplateReceiveTypeDetailEnt> detailEnts = new ArrayList<>();
            List<SysInformOperateTemplateEnt> entitys = new ArrayList<>();
            for (String workflowRelId : workflowRelIds) {
                SysInformOperateTemplateEnt entity = new SysInformOperateTemplateEnt();
                XCopyUtils.copyObject(input, entity);
                String templateId = UUID.randomUUID().toString();
                PpsUserSession session = context.getSession(PpsUserSession.class);
                entity.setId(templateId);
                entity.setFormWorkflowRelId(workflowRelId);
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
                entity.setCreateTime(new Date());
                entitys.add(entity);

                List<CreateSysInformTemplateReceiveTypeInput> typeInputs = input.getReceiveTypeList();
                if (!CollectionUtils.isEmpty(typeInputs)){
                    for (CreateSysInformTemplateReceiveTypeInput typeInput : typeInputs) {
                        typeInput.setInformOperateTemplateId(templateId);
                        String typeId = UUID.randomUUID().toString();
                        typeInput.setId(typeId);
                        SysInformTemplateReceiveTypeEnt typeEnt = new SysInformTemplateReceiveTypeEnt();
                        XCopyUtils.copyObject(typeInput,typeEnt);
                        typeEnts.add(typeEnt);

                        List<CreateSysInformTemplateReceiveTypeDetailInput> typeDetailInputs = typeInput.getReceiveTypeDetailList();
                        if (!CollectionUtils.isEmpty(typeDetailInputs)){
                            for (CreateSysInformTemplateReceiveTypeDetailInput typeDetailInput : typeDetailInputs) {
                                typeDetailInput.setId(UUID.randomUUID().toString());
                                typeDetailInput.setInformOperateTemplateId(templateId);
                                typeDetailInput.setInformTemplateRecieveTypeId(typeId);
                                SysInformTemplateReceiveTypeDetailEnt detailEnt = new SysInformTemplateReceiveTypeDetailEnt();
                                XCopyUtils.copyObject(typeDetailInput,detailEnt);
                                detailEnts.add(detailEnt);
                            }
                        }
                    }
                }
            }

            if (!CollectionUtils.isEmpty(entitys)){
                try {
                    FormWorkflowRelCloudService workflowRelCloudService = context.getBean(FormWorkflowRelCloudService.class);
                    GetFormWorkflowRelInput workflowRelInput = new GetFormWorkflowRelInput();
                    workflowRelInput.setIds(workflowRelIds);
                    workflowRelInput.setInform(true);
                    XServiceResult xServiceResult = workflowRelCloudService.updateFormWorkflowRel(context, workflowRelInput);
                    xServiceResult.throwIfFail();
                }catch (Exception e){
                    throw new XServiceException(e);
                }

                entitys.forEach(mapper::insert);

                if (!CollectionUtils.isEmpty(typeEnts)){
                    SysInformTemplateReceiveTypeMapper typeMapper = context.getBean(SysInformTemplateReceiveTypeMapper.class);
                    typeEnts.forEach(typeMapper::insert);
                }

                if (!CollectionUtils.isEmpty(detailEnts)){
                    SysInformTemplateReceiveTypeDetailMapper detailMapper = context.getBean(SysInformTemplateReceiveTypeDetailMapper.class);
                    detailEnts.forEach(detailMapper::insert);
                }
            }


            return XServiceResult.OK;
        });

    }

    @XText("修改通知模版")
    @XApiPost
    public XServiceResult updateSysInformOperateTemplate(XContext context, UpdateSysInformOperateTemplateInput input)throws XServiceException{
        return XTransactionHelper.begin(context, () -> {
            SysInformOperateTemplateMapper mapper = context.getBean(SysInformOperateTemplateMapper.class);
            QueryWrapper<SysInformOperateTemplateEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysInformOperateTemplateEnt::getId, input.getId());
            SysInformOperateTemplateEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setModifyTime(new Date());

            //删除接收方式
            SysInformTemplateReceiveTypeMapper typeMapper = context.getBean(SysInformTemplateReceiveTypeMapper.class);
            QueryWrapper<SysInformTemplateReceiveTypeEnt> typeWrapper = new QueryWrapper<>();
            typeWrapper.lambda().eq(SysInformTemplateReceiveTypeEnt::getInformOperateTemplateId, input.getId());
            List<SysInformTemplateReceiveTypeEnt> deleteTypeEnts = typeMapper.selectList(typeWrapper);

            //删除接收方
            SysInformTemplateReceiveTypeDetailMapper detailMapper = context.getBean(SysInformTemplateReceiveTypeDetailMapper.class);
            QueryWrapper<SysInformTemplateReceiveTypeDetailEnt> detailWrapper = new QueryWrapper<>();
            detailWrapper.lambda().eq(SysInformTemplateReceiveTypeDetailEnt::getInformOperateTemplateId, input.getId());
            List<SysInformTemplateReceiveTypeDetailEnt> typeDetailEnts = detailMapper.selectList(detailWrapper);

            if (!CollectionUtils.isEmpty(typeDetailEnts)){
                List<String> ids = typeDetailEnts.stream().map(SysInformTemplateReceiveTypeDetailEnt::getId).collect(Collectors.toList());
                detailMapper.deleteBatchIds(ids);
            }
            if (!CollectionUtils.isEmpty(deleteTypeEnts)){
                List<String> ids = deleteTypeEnts.stream().map(SysInformTemplateReceiveTypeEnt::getId).collect(Collectors.toList());
                typeMapper.deleteBatchIds(ids);
            }

            List<SysInformTemplateReceiveTypeEnt> typeEnts = new ArrayList<>();
            List<SysInformTemplateReceiveTypeDetailEnt> detailEnts = new ArrayList<>();
            List<CreateSysInformTemplateReceiveTypeInput> typeInputs = input.getReceiveTypeList();
            if (!CollectionUtils.isEmpty(typeInputs)){
                for (CreateSysInformTemplateReceiveTypeInput typeInput : typeInputs) {
                    typeInput.setInformOperateTemplateId(entity.getId());
                    String typeId = UUID.randomUUID().toString();
                    typeInput.setId(typeId);
                    SysInformTemplateReceiveTypeEnt typeEnt = new SysInformTemplateReceiveTypeEnt();
                    XCopyUtils.copyObject(typeInput,typeEnt);
                    typeEnts.add(typeEnt);

                    List<CreateSysInformTemplateReceiveTypeDetailInput> typeDetailInputs = typeInput.getReceiveTypeDetailList();
                    if (!CollectionUtils.isEmpty(typeDetailInputs)){
                        for (CreateSysInformTemplateReceiveTypeDetailInput typeDetailInput : typeDetailInputs) {
                            typeDetailInput.setId(UUID.randomUUID().toString());
                            typeDetailInput.setInformOperateTemplateId(entity.getId());
                            typeDetailInput.setInformTemplateRecieveTypeId(typeId);
                            SysInformTemplateReceiveTypeDetailEnt detailEnt = new SysInformTemplateReceiveTypeDetailEnt();
                            XCopyUtils.copyObject(typeDetailInput,detailEnt);
                            detailEnts.add(detailEnt);
                        }
                    }
                }
            }
            if (!CollectionUtils.isEmpty(typeEnts)){
                typeEnts.forEach(typeMapper::insert);

            }
            if (!CollectionUtils.isEmpty(detailEnts)){
                detailEnts.forEach(detailMapper::insert);
            }
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("删除通知模版")
    @XApiPost
    public XServiceResult deleteSysInformOperateTemplate(XContext context, DeleteSysInformOperateTemplateInput input){

        return XTransactionHelper.begin(context, () -> {

            SysInformOperateTemplateMapper mapper = context.getBean(SysInformOperateTemplateMapper.class);
            List<String> formWorkflowRelIds = new ArrayList<>();
            QueryWrapper<SysInformOperateTemplateEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysInformOperateTemplateEnt::getId, input.getId());
            SysInformOperateTemplateEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            formWorkflowRelIds.add(entity.getFormWorkflowRelId());
            //删除接收方式
            SysInformTemplateReceiveTypeMapper typeMapper = context.getBean(SysInformTemplateReceiveTypeMapper.class);
            QueryWrapper<SysInformTemplateReceiveTypeEnt> typeWrapper = new QueryWrapper<>();
            typeWrapper.lambda().eq(SysInformTemplateReceiveTypeEnt::getInformOperateTemplateId, input.getId());
            List<SysInformTemplateReceiveTypeEnt> typeEnts = typeMapper.selectList(typeWrapper);

            //删除接收方
            SysInformTemplateReceiveTypeDetailMapper detailMapper = context.getBean(SysInformTemplateReceiveTypeDetailMapper.class);
            QueryWrapper<SysInformTemplateReceiveTypeDetailEnt> detailWrapper = new QueryWrapper<>();
            detailWrapper.lambda().eq(SysInformTemplateReceiveTypeDetailEnt::getInformOperateTemplateId, input.getId());
            List<SysInformTemplateReceiveTypeDetailEnt> typeDetailEnts = detailMapper.selectList(detailWrapper);

            if (!CollectionUtils.isEmpty(typeDetailEnts)){
                List<String> ids = typeDetailEnts.stream().map(SysInformTemplateReceiveTypeDetailEnt::getId).collect(Collectors.toList());
                detailMapper.deleteBatchIds(ids);
            }
            if (!CollectionUtils.isEmpty(typeEnts)){
                List<String> ids = typeEnts.stream().map(SysInformTemplateReceiveTypeEnt::getId).collect(Collectors.toList());
                typeMapper.deleteBatchIds(ids);
            }

            mapper.deleteById(entity);

            queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(SysInformOperateTemplateEnt::getFormWorkflowRelId,formWorkflowRelIds);
            List<SysInformOperateTemplateEnt> templateEnts = mapper.selectList(queryWrapper);
            if (CollectionUtils.isEmpty(templateEnts)){
                try {
                    FormWorkflowRelCloudService workflowRelCloudService = context.getBean(FormWorkflowRelCloudService.class);
                    GetFormWorkflowRelInput workflowRelInput = new GetFormWorkflowRelInput();
                    workflowRelInput.setIds(formWorkflowRelIds);
                    workflowRelInput.setInform(false);
                    XServiceResult xServiceResult = workflowRelCloudService.updateFormWorkflowRel(context, workflowRelInput);
                    xServiceResult.throwIfFail();
                }catch (Exception e){
                    throw new XServiceException(e);
                }
            }

            return XServiceResult.OK;
        });

    }

    @XText("编辑模版")
    @XApiPost
    public XSingleResult<GetSysInformOperateTemplateOutput> getSysInformOperateTemplate(XContext context, GetSysInformOperateTemplateInput input){
        SysInformOperateTemplateMapper mapper = context.getBean(SysInformOperateTemplateMapper.class);
        SysInformTemplateReceiveTypeMapper typeMapper = context.getBean(SysInformTemplateReceiveTypeMapper.class);
        SysInformTemplateReceiveTypeDetailMapper detailMapper = context.getBean(SysInformTemplateReceiveTypeDetailMapper.class);
        QueryWrapper<SysInformOperateTemplateEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysInformOperateTemplateEnt::getId, input.getId());
        SysInformOperateTemplateEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysInformOperateTemplateOutput output = new GetSysInformOperateTemplateOutput();
        XCopyUtils.copyObject(entity, output);

        QueryWrapper<SysInformTemplateReceiveTypeEnt> typeWrapper = new QueryWrapper<>();
        typeWrapper.lambda().eq(SysInformTemplateReceiveTypeEnt::getInformOperateTemplateId,input.getId()).orderBy(true,true,SysInformTemplateReceiveTypeEnt::getId);
        List<SysInformTemplateReceiveTypeEnt> typeEnts = typeMapper.selectList(typeWrapper);
        List<GetSysInformTemplateReceiveTypeOutput> typeOutputs = XCopyUtils.copyNewList(typeEnts, GetSysInformTemplateReceiveTypeOutput.class);
        List<String> typeIds = typeEnts.stream().map(SysInformTemplateReceiveTypeEnt::getId).collect(Collectors.toList());

        QueryWrapper<SysInformTemplateReceiveTypeDetailEnt> typeDetailWrapper = new QueryWrapper<>();
        typeDetailWrapper.lambda().in(SysInformTemplateReceiveTypeDetailEnt::getInformTemplateRecieveTypeId,typeIds);
        List<SysInformTemplateReceiveTypeDetailEnt> typeDetailEnts = detailMapper.selectList(typeDetailWrapper);
        List<GetSysInformTemplateReceiveTypeDetailOutput> detailOutputs = XCopyUtils.copyNewList(typeDetailEnts, GetSysInformTemplateReceiveTypeDetailOutput.class);
        for (GetSysInformTemplateReceiveTypeOutput typeOutput : typeOutputs) {
            List<GetSysInformTemplateReceiveTypeDetailOutput> list = new ArrayList<>();
            for (GetSysInformTemplateReceiveTypeDetailOutput detailOutput : detailOutputs) {
                if (typeOutput.getId().equals(detailOutput.getInformTemplateRecieveTypeId())){
                    list.add(detailOutput);
                }
            }
            typeOutput.setReceiveTypeDetailList(list);
        }
        output.setReceiveTypeList(typeOutputs);
        return XSingleResult.success(output);
    }








}
