package pps.core.system.service;

import java.io.InputStream;
import java.util.*;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.sms.service.SmsService;
import pps.cloud.sms.service.data.MessageParam;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_send_message.*;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XDateUtils;
import xstartup.data.*;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@XText("发送短信管理")
@XService
public class SysSendMessageService {
    @XText("添加发送短信")
    @XApiPost
    public XServiceResult createSysSendMessage(XContext context, CreateSysSendMessageInput input){
        SysSendMessageMapper mapper = context.getBean(SysSendMessageMapper.class);
        SysSendMessageEnt entity = new SysSendMessageEnt();
        SmsService smsService = context.getBean(SmsService.class);
        XCopyUtils.copyObject(input, entity);
        Date date = new Date();
        String uuid = UUID.randomUUID().toString();
        entity.setMessageId(uuid);
        entity.setCreateTime(date);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (session != null){
            entity.setSendUserId(session.getId());
            entity.setSendUserName(session.getUserName());
            entity.setOuId(session.getOuId());
            entity.setOuName(session.getOuName());
        }

        MessageParam param = new MessageParam();
        param.setVariable(entity.getSendContent());
        param.setPhoneNum(entity.getReceiverAccount());
        try {
            XServiceResult xServiceResult = smsService.sendMessage(context, param);
            if(xServiceResult.isSuccess()){
                entity.setStatus("发送成功");
            }
            else {
                entity.setStatus("发送失败");
            }
        } catch (Exception e) {
            entity.setStatus("发送失败");
        }
        Date date1=new Date();
        entity.setRealSendDate(date1);
        entity.setReceiverAccount(UserInfoUtil.encrypt(entity.getReceiverAccount()));
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("批量发送短信")
    @XApiPost
    public XServiceResult insertBatchSysSendMessage(XContext context, CreateSysSendMessageBatchInput input){
        SysSendMessageMapper mapper = context.getBean(SysSendMessageMapper.class);
        SmsService smsService = context.getBean(SmsService.class);
        SysSendMessageEnt entity = null;
        try {
            for (CreateSysSendMessageInput item:input.getListMessage()) {
                entity = new SysSendMessageEnt();
                XCopyUtils.copyObject(item, entity);
                Date date = new Date();
                String uuid = UUID.randomUUID().toString();

                entity.setMessageId(uuid);
                entity.setCreateTime(date);
                PpsUserSession session = context.getSession(PpsUserSession.class);
                if (session != null){
                    entity.setSendUserId(session.getId());
                    entity.setSendUserName(session.getUserName());
                    entity.setOuId(session.getOuId());
                    entity.setOuName(session.getOuName());
                }
                MessageParam param = new MessageParam();
                param.setVariable(entity.getSendContent());
                param.setPhoneNum(entity.getReceiverAccount());
                try {
                    XServiceResult xServiceResult = smsService.sendMessage(context, param);
                    if(xServiceResult.isSuccess()){
                        entity.setStatus("发送成功");
                    }
                    else {
                        entity.setStatus("发送失败");
                    }
                } catch (Exception e) {
                    entity.setStatus("发送失败");
                }
                Date date1=new Date();
                entity.setRealSendDate(date1);
                entity.setReceiverAccount(UserInfoUtil.encrypt(entity.getReceiverAccount()));
                mapper.insert(entity);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XServiceResult.OK;
    }

    @XText("修改发送短信")
    @XApiPost
    public XServiceResult updateSysSendMessage(XContext context, CreateSysSendMessageInput input){
        SysSendMessageMapper mapper = context.getBean(SysSendMessageMapper.class);
        QueryWrapper<SysSendMessageEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSendMessageEnt::getMessageId, input.getMessageId());
        SysSendMessageEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除发送短信")
    @XApiPost
    public XServiceResult deleteSysSendMessage(XContext context, CreateSysSendMessageInput input){
        SysSendMessageMapper mapper = context.getBean(SysSendMessageMapper.class);
        QueryWrapper<SysSendMessageEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSendMessageEnt::getMessageId, input.getMessageId());
        SysSendMessageEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }


    @XApiGet
    public XSingleResult<CreateSysSendMessageOutput> getSysSendMessage(XContext context, CreateSysSendMessageInput input){
        SysSendMessageMapper mapper = context.getBean(SysSendMessageMapper.class);
        QueryWrapper<SysSendMessageEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSendMessageEnt::getMessageId, input.getMessageId());
        SysSendMessageEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        CreateSysSendMessageOutput output = new CreateSysSendMessageOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("查询短信列表")
    @XApiGet
    public XPageResult<QuerySysSendMessageOutput> querySysSendMessage(XContext context, QuerySysSendMessageInput input){
        SysSendMessageViewMapper mapper = context.getBean(SysSendMessageViewMapper.class);

        SysSendMessageView record = new SysSendMessageView();
        List<SysSendMessageView> list = new ArrayList<>();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());

        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (session != null){
            record.setSendUserName(session.getUserName());
        }

        list = mapper.selectList(record);
        list.forEach(item->{
           item.setReceiverAccount(UserInfoUtil.decrypt(item.getReceiverAccount()));
        });

        PageInfo<SysSendMessageView> pageInfo = new PageInfo<>(list);
        List<QuerySysSendMessageOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysSendMessageOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("短信列表-查询-导出")
    @XApiAnonymous
    @XApiGet
    public XFileResult exportQueryExcel(XContext context, EQQuerySysSendMessageInput input) {
        SysSendMessageViewMapper mapper = context.getBean(SysSendMessageViewMapper.class);

        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/发送短信列表_" + XDateUtils.getString(new Date(),"yyyy-MM-dd")+".xlsx");
        String templateFilePath = "template/exportSendSms.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
        SysSendMessageView record = new SysSendMessageView();
        List<SysSendMessageView> list = new ArrayList<>();
        XCopyUtils.copyObject(input, record);
//        PpsUserSession session = context.getSession(PpsUserSession.class);
//        if (session != null){
//            record.setSendUserName(session.getUserName());
//        }
        list = mapper.selectList(record);
        list.forEach(item->{
            item.setReceiverAccount(UserInfoUtil.decrypt(item.getReceiverAccount()));
        });

        Map<String,Object> params = new HashMap<>();
        params.put("list", list);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
        try {
            ExportExcelUtils.outputFileData(excelContent,fileSavePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX,fileSavePath);
    }

}
