package pps.core.system.utils;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import pps.cloud.base.service.BaseBaseOperationalConstraintCloudService;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintInput;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintOuput;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.GetSysUserRoleRelOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.NamedServiceProvider;
import pps.core.system.servicehelper.DataServiceHelper;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

public class CustomWorkflowBehaviour {

    public static final int WorkflowState_Save=0;//暂存
    public static final int WorkflowState_Submit=1;//填报提交
    public static final int WorkflowState_Resubmit=21;//再次提交
    public static final int WorkflowState_Pass=2;//审批通过
    public static final int WorkflowState_Reject =3;//审批退回
    public static final int WorkflowState_RejectSave =30;//审批退回暂存
    public static final int WorkflowState_Back=4;//撤回
    public static final int WorkflowState_BackSave =40;//撤回暂存
    public static final int WorkflowState_ChangeSubmit=11;//申请变更
    public static final int WorkflowState_ChangePass=12;//变更审批通过
    public static final int WorkflowState_ChangeReject=13;//变更审批驳回
    public static final int WorkflowState_ChangeBack=14;//变更申请撤回

    private static final CustomWorkflowButtom saveButtom=new CustomWorkflowButtom("暂存","暂存",WorkflowState_Save);
    private static final CustomWorkflowButtom resetButton=new CustomWorkflowButtom("重置","重置",-1);
    private static final CustomWorkflowButtom sumitButtom=new CustomWorkflowButtom("提交","提交",WorkflowState_Submit);
    private static final CustomWorkflowButtom passButtom=new CustomWorkflowButtom("同意","同意",WorkflowState_Pass);
    private static final CustomWorkflowButtom rejectButtom=new CustomWorkflowButtom("退回","退回", WorkflowState_Reject);
    private static final CustomWorkflowButtom rejectSaveButtom=new CustomWorkflowButtom("暂存","暂存",WorkflowState_RejectSave);
    private static final CustomWorkflowButtom backButtom=new CustomWorkflowButtom("撤回","撤回",WorkflowState_Back);
    private static final CustomWorkflowButtom backSaveButtom=new CustomWorkflowButtom("暂存","暂存", WorkflowState_BackSave);
    private static final CustomWorkflowButtom resubmitButtom=new CustomWorkflowButtom("提交","提交",WorkflowState_Resubmit);
    private static final CustomWorkflowButtom changeSubmitButtom=new CustomWorkflowButtom("申请变更","申请变更",WorkflowState_ChangeSubmit);
    private static final CustomWorkflowButtom changePassButtom=new CustomWorkflowButtom("变更通过","同意",WorkflowState_ChangePass);
    private static final CustomWorkflowButtom changeRejectButtom=new CustomWorkflowButtom("变更驳回","退回", WorkflowState_ChangeReject);
    private static final CustomWorkflowButtom changeBackButtom=new CustomWorkflowButtom("变更撤回","撤回", WorkflowState_ChangeBack);

    private String bsnsType;
    private String bsnsTypeChange;
    private String extendName;
    private String extendNameChange;
    private String processCode;
    private String processCodeChange;

    private Integer formSate;
    private String formId;
    private CustomWorkflowInfor inputInfor;


    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType)
    {
        this.init(processCode,extendName,bsnsType,"","","",null,"",null);
    }
    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType,String processCodeChange,String extendNameChange,String bsnsTypeChange)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,null,"",null);
    }

    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType,Integer formState,String formId)
    {
        this.init(processCode,extendName,bsnsType,"","","",formState,formId,null);
    }
    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType,String processCodeChange,String extendNameChange,String bsnsTypeChange,Integer formState,String formId)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,formState,formId,null);
    }

    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType,CustomWorkflowInfor inputInfor)
    {
        this.init(processCode,extendName,bsnsType,"","","",inputInfor.getFormState(),inputInfor.getFormId(),inputInfor);
    }
    public CustomWorkflowBehaviour(String processCode,String extendName,String bsnsType,String processCodeChange,String extendNameChange,String bsnsTypeChange,CustomWorkflowInfor inputInfor)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,inputInfor.getFormState(),inputInfor.getFormId(),inputInfor);
    }

    private void init(String processCode,String extendName,String bsnsType,String processCodeChange,String extendNameChange,String bsnsTypeChange,Integer formSate,String formId,CustomWorkflowInfor inputInfor)
    {
        this.processCode=processCode;
        this.extendName=extendName;
        this.bsnsType=bsnsType;

        this.processCodeChange=processCodeChange;
        this.extendNameChange=extendNameChange;
        this.bsnsTypeChange=bsnsTypeChange;

        this.formSate=formSate;
        this.formId = formId;
        this.inputInfor=inputInfor;
    }


    public boolean isNeedUpdateFromData() {
        boolean result = false;

        switch (this.formSate) {
            case WorkflowState_Save:
            case WorkflowState_Submit:
            case WorkflowState_Resubmit:
            case WorkflowState_BackSave:
            case WorkflowState_RejectSave:
                result = true;
                break;
            case WorkflowState_ChangeSubmit:
            case WorkflowState_ChangePass:
            case WorkflowState_Pass:
            case WorkflowState_Reject:
            case WorkflowState_Back:
            case WorkflowState_ChangeReject:
            case WorkflowState_ChangeBack:
                result = false;
                break;

        }
        return result;
    }
    public boolean isCanEditFrom() {
        boolean result = false;

        if(this.formSate==null)
            return true;

        switch (this.formSate) {
            case WorkflowState_Save:
            case WorkflowState_BackSave:
            case WorkflowState_RejectSave:
            case WorkflowState_Pass:
            case WorkflowState_Reject:
            case WorkflowState_Back:
                result = true;
                break;
            case WorkflowState_Resubmit:
            case WorkflowState_ChangeSubmit:
            case WorkflowState_Submit:
            case WorkflowState_ChangePass:
            case WorkflowState_ChangeReject:
                result = false;
                break;

        }
        return result;
    }

    public boolean getIsEdit(XContext context){
        boolean isEdit = false;
        HashMap<String,String> map=this.getUsingInforForButtonList();
        String tempProcessCode=map.get("processCode");
        String tempBsnsType=map.get("bsnsType");

        if(this.formSate!=null)
        {
            switch (this.formSate) {
                case WorkflowState_Save:
                    isEdit = true;
                    break;
                case WorkflowState_Resubmit:
                case WorkflowState_Submit:
                    break;
                case WorkflowState_ChangeSubmit:
                    break;
                case WorkflowState_Pass:
                    /*if(checkInRole(context,tempProcessCode)) {
                        isEdit = true;
                    }*/
                    break;
                case WorkflowState_ChangePass:
                    if(checkInRole(context,tempProcessCode)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_RejectSave:
                case WorkflowState_Reject:
                    if(checkHasTodo(context,this.formId,tempBsnsType)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_ChangeReject:
                    break;
                case WorkflowState_BackSave:
                case WorkflowState_Back:
                    if(checkHasTodo(context,this.formId,tempBsnsType)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_ChangeBack:
                    break;
            }
        }else
        {
            if(checkInRole(context,tempProcessCode)) {
                isEdit = true;
            }
        }
        //有提报角色，才能编辑
        if(isEdit){
            isEdit = checkInRole(context,tempProcessCode);
        }
        return isEdit;
    }

    public static Integer getCommonWorkflowState(Integer formSate)
    {
        Integer result=0;
        if(formSate!=null)
        {
            switch (formSate) {
                case WorkflowState_ChangePass:
                case WorkflowState_Save:
                case WorkflowState_RejectSave:
                case WorkflowState_BackSave:
                case WorkflowState_Back:
                    result=1;
                    break;
                case WorkflowState_Submit:
                case WorkflowState_Resubmit:
                    result=3;
                    break;
                case WorkflowState_Pass:
                    result=4;
                    break;
                case WorkflowState_Reject:
                    result=5;
                    break;
                case WorkflowState_ChangeSubmit:
                case WorkflowState_ChangeReject:
                case WorkflowState_ChangeBack:
                    result=4;
                    break;
            }
        }

        return result;
    }

    public List<CustomWorkflowButtom> getButtomList(XContext context, String createById,Date constraintDate,String constraintBusinessType,String stationId)
    {
        List<CustomWorkflowButtom> result=new ArrayList<>();
        HashMap<String,String> map=this.getUsingInforForButtonList();
        String tempProcessCode=map.get("processCode");
        String tempBsnsType=map.get("bsnsType");
        GetBaseOperationalConstraintOuput operationalConstraint = this.getOperationalConstraint(context, constraintDate, constraintBusinessType,stationId);
        if(this.formSate!=null)
        {
            switch (this.formSate) {
                case WorkflowState_ChangePass:
                case WorkflowState_Save:
                    if(operationalConstraint.getCanSaveOrSubmit() && checkInRole(context,tempProcessCode)) {
                        result.add(saveButtom);
                        result.add(resetButton);
                        result.add(sumitButtom);
                    }
                    break;
                case WorkflowState_Submit:
                case WorkflowState_Resubmit:
                    if(operationalConstraint.getCanSaveOrSubmit() && checkCurrentUser(context,createById)){
                        result.add(backButtom);
                    }else if(operationalConstraint.getCanAudit() && checkHasTodo(context,this.formId,tempBsnsType)) {
                        result.add(rejectButtom);
                        result.add(passButtom);
                    }
                    break;
                case WorkflowState_ChangeSubmit:
                    if(operationalConstraint.getCanApplyChange() && checkCurrentUser(context,createById)) {
                        result.add(changeBackButtom);
                    }else if(operationalConstraint.getCanAudit() && checkHasTodo(context,this.formId,tempBsnsType)) {
                        result.add(changeRejectButtom);
                        result.add(changePassButtom);
                    }
                    break;
                case WorkflowState_Pass:
                    if(operationalConstraint.getCanApplyChange() && checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
                case WorkflowState_Reject:
                case WorkflowState_RejectSave:
                    if(operationalConstraint.getCanSaveOrSubmit() && checkInRole(context,tempProcessCode)) {
                        result.add(rejectSaveButtom);
                        result.add(resetButton);
                        result.add(resubmitButtom);
                    }
                    break;
                case WorkflowState_Back:
                case WorkflowState_BackSave:
                    if(operationalConstraint.getCanSaveOrSubmit() && checkInRole(context,tempProcessCode)) {
                        result.add(backSaveButtom);
                        result.add(resetButton);
                        result.add(resubmitButtom);
                    }
                    break;
                case WorkflowState_ChangeReject:
                    if(operationalConstraint.getCanApplyChange() && checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
                case WorkflowState_ChangeBack:
                    if(!operationalConstraint.getCanApplyChange()) break;
                    if(checkCurrentUser(context,createById)) {
                        result.add(changeSubmitButtom);
                    }else if(checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
            }
        }
        else
        {
            if(operationalConstraint.getCanSaveOrSubmit() && checkInRole(context,tempProcessCode)) {
                result.add(saveButtom);
                result.add(resetButton);
                result.add(sumitButtom);
            }
        }
        context.getLogger().info("===get_buttoms,formSate: {},tempProcessCode: {}, createById: {}",formSate,tempProcessCode,createById);
        return result;
    }
    
    private GetBaseOperationalConstraintOuput getOperationalConstraint(XContext context,Date constraintDate,String constraintBusinessType,
                                                                       String stationId){
        if(constraintDate == null) constraintDate = new Date();
        BaseBaseOperationalConstraintCloudService constraintService = NamedServiceProvider.getServiceByName(context, DataServiceHelper.BASE_INFO_SERVICE_NAME, BaseBaseOperationalConstraintCloudService.class);
        GetBaseOperationalConstraintInput  input = new GetBaseOperationalConstraintInput();
        input.setBusineType(constraintBusinessType);
        input.setOperationalDate(constraintDate);
        input.setStationId(stationId);
        XSingleResult<GetBaseOperationalConstraintOuput> result = constraintService.getBaseOperationalConstraint(context, input);
        GetBaseOperationalConstraintOuput output = null;
        if(result.isSuccess()){
            output = result.getResult();
        }else{
            output = new GetBaseOperationalConstraintOuput();
            output.setCanSaveOrSubmit(true);
            output.setCanApplyChange(true);
            output.setCanAudit(true);
        }
        return output;
    }

    /**
     * 校验上一步工作流是否是当前用户操作的（true 是 false 否）
     * @param context
     * @param createById
     * @return
     */
    private boolean checkCurrentUser(XContext context,String createById){
        boolean result = false;
        if(StringUtils.isNotBlank(createById)){
            PpsUserSession session = context.getSession(PpsUserSession.class);
            result = session.getId().equals(createById);
        }
        return result;
    }

    public CustomWorkflowInfor submitToWorkflow(XContext context)
    {
        try {
            BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
            HashMap<String,String> map=this.getUsingInforForSubmit();
            String tempProcessCode=map.get("processCode");
            String tempBsnsType=map.get("bsnsType");
            String tempExtendName=map.get("extendName");


            XServiceResult xServiceResult = null;
            switch (this.formSate) {
                case WorkflowState_Save:
                case WorkflowState_RejectSave:
                case WorkflowState_BackSave:
                    inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Submit:
                case WorkflowState_ChangeSubmit:
                    StartProcessInput startProcessInput = new StartProcessInput();
                    startProcessInput.setProcessCode(tempProcessCode);
                    startProcessInput.setBsnsDetailUrl(this.inputInfor.getBsnsDetailUrl());
                    startProcessInput.setBsnsType(tempBsnsType);
                    startProcessInput.setExtendName(tempExtendName);
                    startProcessInput.setBsnsId(this.formId);
                    XSingleResult serviceResult = bpmService.startProcess(context, startProcessInput);
                    serviceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Resubmit:
                    FinishWorkItemDirectlyInput finishWorkItemDirectlyInput = new FinishWorkItemDirectlyInput();
                    finishWorkItemDirectlyInput.setProcessCode(tempProcessCode);
                    finishWorkItemDirectlyInput.setBsnsId(this.formId);
                    xServiceResult = bpmService.approvalPass(context, finishWorkItemDirectlyInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Pass:
                    FinishWorkItemDirectlyInput fwidi = new FinishWorkItemDirectlyInput();
                    fwidi.setProcessCode(tempProcessCode);
                    fwidi.setRemark(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    fwidi.setBsnsId(this.formId);
                    xServiceResult = bpmService.approvalPass(context, fwidi);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    this.inputInfor.setPass(true);
                    break;
                case WorkflowState_ChangePass:
                    FinishWorkItemDirectlyInput fwidiChangePass = new FinishWorkItemDirectlyInput();
                    fwidiChangePass.setProcessCode(tempProcessCode);
                    fwidiChangePass.setRemark(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    fwidiChangePass.setBsnsId(this.formId);
                    xServiceResult = bpmService.approvalPass(context, fwidiChangePass);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Reject:
                    ApprovalRejectInput approvalRejectInput = new ApprovalRejectInput();
                    approvalRejectInput.setProcessCode(tempProcessCode);
                    approvalRejectInput.setBackReason(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    approvalRejectInput.setBsnsId(this.formId);
                    xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_ChangeReject:
                    //结束流程
                    TerminateWorkItemInput rejectTerminateInput = new TerminateWorkItemInput();
                    rejectTerminateInput.setProcessCode(tempProcessCode);
                    rejectTerminateInput.setBsnsId(this.formId);
                    rejectTerminateInput.setRemark(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    XServiceResult rejectTerminateResult = bpmService.approvalStop(context, rejectTerminateInput);
                    rejectTerminateResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Back:
                    BackWorkItemInput backInput = new BackWorkItemInput();
                    backInput.setBsnsId(this.formId);
                    backInput.setProcessCode(tempProcessCode);
                    xServiceResult = bpmService.approvalBack(context, backInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_ChangeBack:
                    BackWorkItemInput changeBackInput = new BackWorkItemInput();
                    changeBackInput.setBsnsId(this.formId);
                    changeBackInput.setProcessCode(tempProcessCode);
                    xServiceResult = bpmService.approvalBack(context, changeBackInput);
                    xServiceResult.throwIfFail();
                    //结束流程
                    TerminateWorkItemInput backTerminateInput = new TerminateWorkItemInput();
                    backTerminateInput.setProcessCode(tempProcessCode);
                    backTerminateInput.setBsnsId(this.formId);
                    backTerminateInput.setRemark("变更流程撤回，终止流程");
                    XServiceResult backTerminateResult = bpmService.approvalStop(context, backTerminateInput);
                    backTerminateResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
            }

        }
        catch (Exception ex)
        {
            this.inputInfor.setSubmitSuccess(false);
            this.inputInfor.setSubmitResultMessage(ex.toString());
        }

        return this.inputInfor;
    }

    private HashMap<String,String> getUsingInforForSubmit()
    {
        HashMap<String,String> result=new HashMap<>();
        if(this.formSate!=null)
        {
            if(this.inputInfor.getFromName()==null)
                this.inputInfor.setFromName("");

            if(this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Submit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Resubmit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Reject)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_RejectSave)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Back)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_BackSave)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Save)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Pass)
            )
            {
                result.put("processCode",this.processCode);
                result.put("bsnsType",this.bsnsType);
                result.put("extendName",this.extendName+ this.inputInfor.getFromName());
            }
            else  if(this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeSubmit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangePass)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeReject)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeBack)
            )
            {
                result.put("processCode",this.processCodeChange);
                result.put("bsnsType",this.bsnsTypeChange);
                result.put("extendName",this.extendNameChange+ this.inputInfor.getFromName());
            }
        }
        else
        {
            result.put("processCode","");
            result.put("bsnsType","");
            result.put("extendName","");
        }
        return result;
    }

    private HashMap<String,String> getUsingInforForButtonList()
    {
        HashMap<String,String> result=new HashMap<>();
        if(this.formSate!=null)
        {
            if(this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Submit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Resubmit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Reject)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_RejectSave)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Back)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_BackSave)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Save)
            )
            {
                result.put("processCode",this.processCode);
                result.put("bsnsType",this.bsnsType);
                result.put("extendName",this.extendName);
            }
            else  if(this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeSubmit)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangePass)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeReject)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_ChangeBack)
                    ||this.formSate.equals(CustomWorkflowBehaviour.WorkflowState_Pass))
            {
                result.put("processCode",this.processCodeChange);
                result.put("bsnsType",this.bsnsTypeChange);
                result.put("extendName",this.extendNameChange);
            }
        }
        else
        {
            result.put("processCode",this.processCode);
            result.put("bsnsType",this.bsnsType);
            result.put("extendName",this.extendName);
        }
        return result;
    }

    private static boolean checkHasTodo(XContext context,String bsnsId,String bsnsType){
        if(StringUtils.isBlank(bsnsId) || StringUtils.isBlank(bsnsType))
            return false;
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);
        CheckHasTodoInput chti = new CheckHasTodoInput();
        chti.setBsnsId(bsnsId);
        chti.setBsnsType(bsnsType);
        XSingleResult<CheckHasTodoOutput> result = processService.checkHasTodo(context, chti);
        result.throwIfFail();
        return result.getResult().getHasTodo();
    }
    private static boolean checkInRole(XContext context,String processCode){

        boolean result=false;
        SystemUserRoleRelService systemUserRoleRelService = context.getBean(SystemUserRoleRelService.class);
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);

        GetProcessFirstRoleInput chti = new GetProcessFirstRoleInput();
        chti.setProcessCode(processCode);
        XListResult<GetProcessFirstRoleOutput> rrList = processService.getProcessFirstRoles(context, chti);

        GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
        getSysUserRoleRelInput.setUserId(session.getId());

        List<GetSysUserRoleRelOutput> getSysUserRoleRelOutputs = systemUserRoleRelService.getSysRoleByUserId(context , getSysUserRoleRelInput).getResult();
        for (int i = 0; i < getSysUserRoleRelOutputs.size(); i++) {
            GetSysUserRoleRelOutput getSysUserRoleRelOutput =  getSysUserRoleRelOutputs.get(i);
            result = rrList.getResult().stream().anyMatch(c->c.getRoleId().equals(getSysUserRoleRelOutput.getRoleId()));
            if(result)
                break;
        }


        return result;
    }

    /**
     * 删除工作工作流信息
     * @param context
     */
    public static void deleteProcessInstance(XContext context,String bsnsId){
        if(Strings.isBlank(bsnsId)) return;
        BpsWorkFlowProcessService bpsWorkFlowProcessService = context.getBean(BpsWorkFlowProcessService.class);
        DeleteProcessInstanceInput deleteProcessInstanceInput = new DeleteProcessInstanceInput();
        deleteProcessInstanceInput.setBsnsId(bsnsId);
        XServiceResult deleteResult = bpsWorkFlowProcessService.deleteProcessInstance(context, deleteProcessInstanceInput);
    }

}
