package pps.core.system.utils;

import org.apache.commons.lang3.StringUtils;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.GetSysUserRoleRelOutput;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@XText("LNG装车日报工作流相关")
public class LngLoadRecordWorkflowBehaviour {

    public static final int WorkflowState_Save=0;//暂存
    public static final int WorkflowState_Submit=1;//填报提交
    public static final int WorkflowState_ChangeSubmit=5;//申请变更
    public static final int WorkflowState_Pass=3;//审批通过
    public static final int WorkflowState_ChangePass=7;//变更审批通过
    public static final int WorkflowState_Reject =4;//审批退回
    public static final int WorkflowState_ChangeReject =8;//变更申请审核驳回
    public static final int WorkflowState_ChangeBack=6;//变更审批撤回
    public static final int WorkflowState_Back=2;//撤回

    public static final int WORKFLOWTYPE_SAVE = 4;
    public static final int WORKFLOWTYPE_SUBMIT = 1;
    public static final int WORKFLOWTYPE_REJECT = 2;
    public static final int WORKFLOWTYPE_BACK = 3;




    private static final CustomWorkflowButtom saveButtom=new CustomWorkflowButtom("暂存",WorkflowState_Save,WORKFLOWTYPE_SAVE);
    private static final CustomWorkflowButtom sumitButtom=new CustomWorkflowButtom("提交",WorkflowState_Submit,WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom passButtom=new CustomWorkflowButtom("同意",WorkflowState_Pass,WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom rejectButtom=new CustomWorkflowButtom("退回", WorkflowState_Reject,WORKFLOWTYPE_REJECT);
    private static final CustomWorkflowButtom backButtom=new CustomWorkflowButtom("撤回",WorkflowState_Back,WORKFLOWTYPE_BACK);
    private static final CustomWorkflowButtom changeSubmitButtom=new CustomWorkflowButtom("申请变更",WorkflowState_ChangeSubmit,WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom changePassButtom=new CustomWorkflowButtom("同意变更",WorkflowState_ChangePass,WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom changeRejectButtom=new CustomWorkflowButtom("变更驳回", WorkflowState_ChangeReject,WORKFLOWTYPE_REJECT);
    private static final CustomWorkflowButtom changeBackButtom=new CustomWorkflowButtom("变更撤回", WorkflowState_ChangeBack,WORKFLOWTYPE_BACK);
    private String bsnsType;
    private String bsnsTypeChange;
    private String extendName;
    private String extendNameChange;
    private String processCode;
    private String processCodeChange;

    private Integer state;
    private String bsnsId;
    private LngLoadRecordWorkflowInfor inputInfor;


    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType)
    {
        this.init(processCode,extendName,bsnsType,"","","",null,"",null);
    }
    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType, String processCodeChange, String extendNameChange, String bsnsTypeChange)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,null,"",null);
    }

    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType, Integer formState, String bsnsId)
    {
        this.init(processCode,extendName,bsnsType,"","","",formState,bsnsId,null);
    }
    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType, String processCodeChange, String extendNameChange, String bsnsTypeChange, Integer formState, String bsnsId)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,formState,bsnsId,null);
    }

    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType, LngLoadRecordWorkflowInfor inputInfor)
    {
        this.init(processCode,extendName,bsnsType,"","","",inputInfor.getState(),inputInfor.getBsnsId(),inputInfor);
    }
    public LngLoadRecordWorkflowBehaviour(String processCode, String extendName, String bsnsType, String processCodeChange, String extendNameChange, String bsnsTypeChange, LngLoadRecordWorkflowInfor inputInfor)
    {
        this.init(processCode,extendName,bsnsType,processCodeChange,extendNameChange,bsnsTypeChange,inputInfor.getState(),inputInfor.getBsnsId(),inputInfor);
    }

    private void init(String processCode,String extendName,String bsnsType,String processCodeChange,String extendNameChange,String bsnsTypeChange,Integer state,String bsnsId,LngLoadRecordWorkflowInfor inputInfor)
    {
        this.processCode=processCode;
        this.extendName=extendName;
        this.bsnsType=bsnsType;

        this.processCodeChange=processCodeChange;
        this.extendNameChange=extendNameChange;
        this.bsnsTypeChange=bsnsTypeChange;

        this.state=state;
        this.bsnsId = bsnsId;
        this.inputInfor=inputInfor;
    }


    public boolean isNeedUpdateFromData() {
        boolean result = false;

        switch (this.state) {
            case WorkflowState_Save:
            case WorkflowState_Submit:
            case WorkflowState_ChangeReject:
                result = true;
                break;
            case WorkflowState_ChangeSubmit:
            case WorkflowState_ChangePass:
            case WorkflowState_Pass:
            case WorkflowState_Reject:
            case WorkflowState_Back:
            case WorkflowState_ChangeBack:
                result = false;
                break;

        }
        return result;
    }
    public boolean isCanEditFrom() {
        boolean result = false;

        if(this.state==null)
            return true;

        switch (this.state) {
            case WorkflowState_Save:
            case WorkflowState_ChangeReject:
            case WorkflowState_Pass:
            case WorkflowState_Reject:
            case WorkflowState_Back:
                result = true;
                break;
            case WorkflowState_ChangeSubmit:
            case WorkflowState_Submit:
            case WorkflowState_ChangePass:
            case WorkflowState_ChangeBack:
                result = false;
                break;

        }
        return result;
    }

    public static Integer getCommonWorkflowState(Integer state)
    {
        Integer result=0;
        if(state!=null)
        {
            switch (state) {
                case WorkflowState_Save:
                case WorkflowState_ChangeReject:
                case WorkflowState_Back:
                    result=1;
                    break;
                case WorkflowState_Submit:
                case WorkflowState_ChangeSubmit:
                    result=3;
                    break;
                case WorkflowState_Pass:
                case WorkflowState_ChangePass:
                case WorkflowState_ChangeBack:
                    result=4;
                    break;
                case WorkflowState_Reject:
                    result=5;
                    break;
            }
        }

        return result;
    }

    public List<CustomWorkflowButtom> getButtomList(XContext context,String createById)
    {
        List<CustomWorkflowButtom> result=new ArrayList<>();
        HashMap<String,String> map=this.getUsingInforForButtonList();
        String tempProcessCode=map.get("processCode");
        String tempBsnsType=map.get("bsnsType");
        String tempExtendName=map.get("extendName");

        if(this.state!=null)
        {
            switch (this.state) {
                case WorkflowState_ChangePass:
                case WorkflowState_Save:
                    if(checkInRole(context,tempProcessCode)){
                        result.add(saveButtom);
                        result.add(sumitButtom);
                    }
                    break;
                case WorkflowState_Submit:
                    if(checkCurrentUser(context,createById)) {
                        result.add(backButtom);
                    }else if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        result.add(passButtom);
                        result.add(rejectButtom);
                    }
                    break;
                case WorkflowState_ChangeSubmit:
                    if(checkCurrentUser(context,createById)) {
                        result.add(changeBackButtom);
                    }else if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        result.add(changePassButtom);
                        result.add(changeRejectButtom);
                    }
                    break;
                case WorkflowState_Pass:
                    if(checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
                case WorkflowState_Reject:
                    if(checkInRole(context,tempProcessCode)) {
                        result.add(saveButtom);
                        result.add(sumitButtom);
                    }
                    break;
                case WorkflowState_ChangeReject:
                    if(checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
                case WorkflowState_Back:
                    if(checkInRole(context,tempProcessCode)) {
                        result.add(saveButtom);
                        result.add(sumitButtom);
                    }
                    break;
                case WorkflowState_ChangeBack:
                    if(checkCurrentUser(context,createById)) {
                        result.add(changeSubmitButtom);
                    }else if(checkInRole(context,tempProcessCode)) {
                        result.add(changeSubmitButtom);
                    }
                    break;
            }
        }
        else
        {
            if(checkInRole(context,tempProcessCode)) {
                result.add(saveButtom);
                result.add(sumitButtom);
            }
        }

        return result;
    }

    /**
     * 校验上一步工作流是否是当前用户操作的（true 是 false 否）
     * @param context
     * @param createById
     * @return
     */
    private boolean checkCurrentUser(XContext context,String createById){
        boolean result = false;
        if(StringUtils.isNotBlank(createById)){
            PpsUserSession session = context.getSession(PpsUserSession.class);
            result = session.getId().equals(createById);
        }
        return result;
    }


    public boolean getIsEdit(XContext context,String createById)
    {
        boolean isEdit = false;
        List<CustomWorkflowButtom> result=new ArrayList<>();
        HashMap<String,String> map=this.getUsingInforForButtonList();
        String tempProcessCode=map.get("processCode");
        String tempBsnsType=map.get("bsnsType");
        String tempExtendName=map.get("extendName");

        if(this.state!=null)
        {
            switch (this.state) {
                case WorkflowState_Save:
                    isEdit = true;
                    break;
                case WorkflowState_Submit:
                    break;
                case WorkflowState_ChangeSubmit:
                    break;
                case WorkflowState_Pass:
                    /*if(checkInRole(context,tempProcessCode)) {
                        isEdit = true;
                    }*/
                    break;
                case WorkflowState_ChangePass:
                    if(checkInRole(context,tempProcessCode)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_Reject:
                    if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_ChangeReject:
                    break;
                case WorkflowState_Back:
                    if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_ChangeBack:
                    break;
            }
        }else
        {
            if(checkInRole(context,tempProcessCode)) {
                isEdit = true;
            }
        }
        //有提报角色，才能编辑
        if(isEdit){
            isEdit = checkInRole(context,tempProcessCode);
        }
        return isEdit;
    }


    public boolean getShowRemark(XContext context,String createById)
    {
        boolean showRemark = false;
        HashMap<String,String> map=this.getUsingInforForButtonList();
        String tempProcessCode=map.get("processCode");
        String tempBsnsType=map.get("bsnsType");

        if(this.state!=null)
        {
            switch (this.state) {
                case WorkflowState_ChangePass:
                case WorkflowState_Save:
                    break;
                case WorkflowState_Submit:
                    if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        showRemark=true;
                    }
                    break;
                case WorkflowState_ChangeSubmit:
                    if(checkHasTodo(context,this.bsnsId,tempBsnsType)) {
                        showRemark=true;
                    }
                    break;
                case WorkflowState_Pass:
                    if(checkInRole(context,tempProcessCode)) {
                        showRemark=true;
                    }
                    break;
                case WorkflowState_Reject:
                    break;
                case WorkflowState_ChangeReject:
                    if(checkInRole(context,tempProcessCode)) {
                        showRemark=true;
                    }
                    break;
                case WorkflowState_Back:
                    break;
                case WorkflowState_ChangeBack:
                    if(checkInRole(context,tempProcessCode)) {
                        showRemark=true;
                    }
                    break;
            }
        }
        return showRemark;
    }

    public boolean getShowWork(XContext context,String createById)
    {
        boolean showWork = false;

        if(this.state!=null)
        {
            switch (this.state) {
                case WorkflowState_Save:
                    break;
                case WorkflowState_Submit:
                case WorkflowState_ChangeSubmit:
                case WorkflowState_Pass:
                case WorkflowState_ChangePass:
                case WorkflowState_Reject:
                case WorkflowState_ChangeReject:
                case WorkflowState_Back:
                case WorkflowState_ChangeBack:
                    showWork =true;
                    break;
            }
        }
        return showWork;
    }

    public LngLoadRecordWorkflowInfor submitToWorkflow(XContext context)
    {
        try {
            BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
            HashMap<String,String> map=this.getUsingInforForSubmit();
            String tempProcessCode=map.get("processCode");
            String tempBsnsType=map.get("bsnsType");
            String tempExtendName=map.get("extendName");


            XServiceResult xServiceResult = null;
            switch (this.state) {
                case WorkflowState_Save:
                case WorkflowState_ChangeReject:
                    inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Submit:
                case WorkflowState_ChangeSubmit:
                    StartProcessInput startProcessInput = new StartProcessInput();
                    startProcessInput.setProcessCode(tempProcessCode);
                    startProcessInput.setBsnsDetailUrl(this.inputInfor.getBsnsDetailUrl());
                    startProcessInput.setBsnsType(tempBsnsType);
                    startProcessInput.setExtendName(tempExtendName);
                    startProcessInput.setBsnsId(this.bsnsId);
                    XSingleResult serviceResult = bpmService.startProcess(context, startProcessInput);
                    serviceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Pass:
                    FinishWorkItemDirectlyInput fwidi = new FinishWorkItemDirectlyInput();
                    fwidi.setProcessCode(tempProcessCode);
                    fwidi.setRemark(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    fwidi.setBsnsId(this.bsnsId);
                    xServiceResult = bpmService.approvalPass(context, fwidi);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    this.inputInfor.setPass(true);
                    break;
                case WorkflowState_ChangePass:
                    FinishWorkItemDirectlyInput fwidiChangePass = new FinishWorkItemDirectlyInput();
                    fwidiChangePass.setProcessCode(tempProcessCode);
                    fwidiChangePass.setRemark(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    fwidiChangePass.setBsnsId(this.bsnsId);
                    xServiceResult = bpmService.approvalPass(context, fwidiChangePass);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Reject:
                    ApprovalRejectInput approvalRejectInput = new ApprovalRejectInput();
                    approvalRejectInput.setProcessCode(tempProcessCode);
                    approvalRejectInput.setBackReason(this.inputInfor.getRemark()==null?"":this.inputInfor.getRemark());
                    approvalRejectInput.setBsnsId(this.bsnsId);
                    xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_ChangeBack:
                    TerminateWorkItemInput terminateWorkItemInput = new TerminateWorkItemInput();
                    terminateWorkItemInput.setBsnsId(this.bsnsId);
                    terminateWorkItemInput.setProcessCode(tempProcessCode);
                    xServiceResult = bpmService.approvalStop(context, terminateWorkItemInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Back:
                    BackWorkItemInput backInput = new BackWorkItemInput();
                    backInput.setBsnsId(this.bsnsId);
                    backInput.setProcessCode(tempProcessCode);
                    xServiceResult = bpmService.approvalBack(context, backInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
            }

        }
        catch (Exception ex)
        {
            this.inputInfor.setSubmitSuccess(false);
            this.inputInfor.setSubmitResultMessage(ex.toString());
        }

        return this.inputInfor;
    }

    private HashMap<String,String> getUsingInforForSubmit()
    {
        HashMap<String,String> result=new HashMap<>();
        if(this.state!=null)
        {
            if(this.inputInfor.getExtendName()==null)
                this.inputInfor.setExtendName("");

            if(this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Submit)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Reject)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeReject)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Back)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Save)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Pass)
            )
            {
                result.put("processCode",this.processCode);
                result.put("bsnsType",this.bsnsType);
                result.put("extendName",this.extendName+ this.inputInfor.getExtendName());
            }
            else  if(this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeSubmit)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangePass)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeBack)
            )
            {
                result.put("processCode",this.processCodeChange);
                result.put("bsnsType",this.bsnsTypeChange);
                result.put("extendName",this.extendNameChange+ this.inputInfor.getExtendName());
            }
        }
        else
        {
            result.put("processCode","");
            result.put("bsnsType","");
            result.put("extendName","");
        }
        return result;
    }

    private HashMap<String,String> getUsingInforForButtonList()
    {
        HashMap<String,String> result=new HashMap<>();
        if(this.state!=null)
        {
            if(this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Submit)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Reject)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeReject)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Back)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Save)
            )
            {
                result.put("processCode",this.processCode);
                result.put("bsnsType",this.bsnsType);
                result.put("extendName",this.extendName);
            }
            else  if(this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeSubmit)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangePass)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_ChangeBack)
                    ||this.state.equals(LngLoadRecordWorkflowBehaviour.WorkflowState_Pass))
            {
                result.put("processCode",this.processCodeChange);
                result.put("bsnsType",this.bsnsTypeChange);
                result.put("extendName",this.extendNameChange);
            }
        }
        else
        {
            result.put("processCode",this.processCode);
            result.put("bsnsType",this.bsnsType);
            result.put("extendName",this.extendName);
        }
        return result;
    }

    private static boolean checkHasTodo(XContext context,String bsnsId,String bsnsType){
        if(StringUtils.isBlank(bsnsId) || StringUtils.isBlank(bsnsType))
            return false;
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);
        CheckHasTodoInput chti = new CheckHasTodoInput();
        chti.setBsnsId(bsnsId);
        chti.setBsnsType(bsnsType);
        XSingleResult<CheckHasTodoOutput> result = processService.checkHasTodo(context, chti);
        result.throwIfFail();
        return result.getResult().getHasTodo();
    }
    
    private static boolean checkInRole(XContext context,String processCode){

        boolean result=false;
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String userId = session.getId();
        SystemUserRoleRelService systemUserRoleRelService = context.getBean(SystemUserRoleRelService.class);
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);

        GetProcessFirstRoleInput chti = new GetProcessFirstRoleInput();
        chti.setProcessCode(processCode);
        XListResult<GetProcessFirstRoleOutput> rrList = processService.getProcessFirstRoles(context, chti);
        if(!rrList.isSuccess() || rrList.getResult() == null) return result;

        GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
        getSysUserRoleRelInput.setUserId(userId);

        List<GetSysUserRoleRelOutput> getSysUserRoleRelOutputs = systemUserRoleRelService.getSysRoleByUserId(context , getSysUserRoleRelInput).getResult();
        for (int i = 0; i < getSysUserRoleRelOutputs.size(); i++) {
            GetSysUserRoleRelOutput getSysUserRoleRelOutput =  getSysUserRoleRelOutputs.get(i);
            result = rrList.getResult().stream().anyMatch(c->c.getRoleId().equals(getSysUserRoleRelOutput.getRoleId()));
            if(result)
                break;
        }
        return result;
    }

}
