package pps.core.system.utils;

import org.apache.commons.lang3.StringUtils;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.GetSysUserRoleRelOutput;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@XText("运维管理工作流相关")
public class OperationManagementWorkflowBehaviour {

    public static final int WorkflowState_Submit = 1;//填报提交
    public static final int WorkflowState_Pass = 3;//审批通过
    public static final int WorkflowState_Reject = 4;//审批退回

    public static final int WORKFLOWTYPE_SUBMIT = 1;
    public static final int WORKFLOWTYPE_REJECT = 2;


    private static final CustomWorkflowButtom sumitButtom = new CustomWorkflowButtom("提交", WorkflowState_Submit, WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom passButtom = new CustomWorkflowButtom("同意", WorkflowState_Pass, WORKFLOWTYPE_SUBMIT);
    private static final CustomWorkflowButtom rejectButtom = new CustomWorkflowButtom("退回", WorkflowState_Reject, WORKFLOWTYPE_REJECT);

    private String bsnsType;
    private String bsnsTypeChange;
    private String extendName;
    private String extendNameChange;
    private String processCode;
    private String processCodeChange;

    private Integer auditstatus;
    private String bsnsId;
    private LngLoadRecordWorkflowInfor inputInfor;


    public String getProcessCode() {
        return processCode;
    }

    public void setProcessCode(String processCode) {
        this.processCode = processCode;
    }

    public Integer getAuditstatus() {
        return auditstatus;
    }

    public void setAuditstatus(Integer auditstatus) {
        this.auditstatus = auditstatus;
    }

    public String getBsnsId() {
        return bsnsId;
    }

    public void setBsnsId(String bsnsId) {
        this.bsnsId = bsnsId;
    }

    private void init(String processCode, String extendName, String bsnsType, String processCodeChange, String extendNameChange, String bsnsTypeChange, Integer state, String bsnsId, LngLoadRecordWorkflowInfor inputInfor) {
        this.processCode = processCode;
        this.extendName = extendName;
        this.bsnsType = bsnsType;

        this.processCodeChange = processCodeChange;
        this.extendNameChange = extendNameChange;
        this.bsnsTypeChange = bsnsTypeChange;

        this.auditstatus = auditstatus;
        this.bsnsId = bsnsId;
        this.inputInfor = inputInfor;
    }


    public boolean isNeedUpdateFromData() {
        boolean result = false;

        switch (this.auditstatus) {
            case WorkflowState_Submit:
                result = true;
                break;
            case WorkflowState_Pass:
            case WorkflowState_Reject:
                result = false;
                break;

        }
        return result;
    }

    public boolean isCanEditFrom() {
        boolean result = false;

        if (this.auditstatus == null)
            return true;

        switch (this.auditstatus) {
            case WorkflowState_Pass:
            case WorkflowState_Reject:
                result = true;
                break;
            case WorkflowState_Submit:
                result = false;
                break;

        }
        return result;
    }

    public static Integer getCommonWorkflowState(Integer auditstatus) {
        Integer result = 0;
        if (auditstatus != null) {
            switch (auditstatus) {
                case WorkflowState_Submit:
                    result = 3;
                    break;
                case WorkflowState_Pass:
                    result = 4;
                    break;
                case WorkflowState_Reject:
                    result = 5;
                    break;
            }
        }

        return result;
    }

    public List<CustomWorkflowButtom> getButtomList(XContext context, String createById) {
        List<CustomWorkflowButtom> result = new ArrayList<>();
        HashMap<String, String> map = this.getUsingInforForButtonList();
        String tempProcessCode = map.get("processCode");
        String tempBsnsType = map.get("bsnsType");

        if (this.auditstatus != null) {
            switch (this.auditstatus) {
                case WorkflowState_Submit:
                    if (checkHasTodo(context, this.bsnsId, tempBsnsType)) {
                        result.add(passButtom);
                        result.add(rejectButtom);
                    }
                    break;
                case WorkflowState_Pass:
                    if (checkInRole(context, tempProcessCode, createById)) {
                        result.add(passButtom);
                    }
                    break;
                case WorkflowState_Reject:
                    if (checkHasTodo(context, this.bsnsId, tempBsnsType)) {
                        result.add(sumitButtom);
                    }
                    break;
            }
        } else {
            if (checkInRole(context, tempProcessCode, createById)) {
                result.add(sumitButtom);
            }
        }

        return result;
    }

    public boolean getIsEdit(XContext context, String createById) {
        boolean isEdit = false;
        HashMap<String, String> map = this.getUsingInforForButtonList();
        String tempProcessCode = map.get("processCode");
        String tempBsnsType = map.get("bsnsType");

        if (this.auditstatus != null) {
            switch (this.auditstatus) {
                case WorkflowState_Submit:
                    break;
                case WorkflowState_Pass:
                    if (checkInRole(context, tempProcessCode, createById)) {
                        isEdit = true;
                    }
                    break;
                case WorkflowState_Reject:
                    if (checkHasTodo(context, this.bsnsId, tempBsnsType)) {
                        isEdit = true;
                    }
                    break;
            }
        } else {
            if (checkInRole(context, tempProcessCode, createById)) {
                isEdit = true;
            }
        }
        return isEdit;
    }


    public boolean getShowRemark(XContext context, String createById) {
        boolean showRemark = false;
        HashMap<String, String> map = this.getUsingInforForButtonList();
        String tempProcessCode = map.get("processCode");
        String tempBsnsType = map.get("bsnsType");

        if (this.auditstatus != null) {
            switch (this.auditstatus) {
                case WorkflowState_Submit:
                    if (checkHasTodo(context, this.bsnsId, tempBsnsType)) {
                        showRemark = true;
                    }
                    break;
                case WorkflowState_Pass:
                    if (checkInRole(context, tempProcessCode, createById)) {
                        showRemark = true;
                    }
                    break;
                case WorkflowState_Reject:
                    break;
            }
        }
        return showRemark;
    }

    public boolean getShowWork(XContext context, String createById) {
        boolean showWork = false;

        if (this.auditstatus != null) {
            switch (this.auditstatus) {
                case WorkflowState_Submit:
                case WorkflowState_Pass:
                case WorkflowState_Reject:
                    showWork = true;
                    break;
            }
        }
        return showWork;
    }

    public LngLoadRecordWorkflowInfor submitToWorkflow(XContext context) {
        try {
            BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
            HashMap<String, String> map = this.getUsingInforForSubmit();
            String tempProcessCode = map.get("processCode");
            String tempBsnsType = map.get("bsnsType");
            String tempExtendName = map.get("extendName");


            XServiceResult xServiceResult = null;
            switch (this.auditstatus) {
                case WorkflowState_Submit:
                    StartProcessInput startProcessInput = new StartProcessInput();
                    startProcessInput.setProcessCode(tempProcessCode);
                    startProcessInput.setBsnsDetailUrl(this.inputInfor.getBsnsDetailUrl());
                    startProcessInput.setBsnsType(tempBsnsType);
                    startProcessInput.setExtendName(tempExtendName);
                    startProcessInput.setBsnsId(this.bsnsId);
                    XSingleResult serviceResult = bpmService.startProcess(context, startProcessInput);
                    serviceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
                case WorkflowState_Pass:
                    FinishWorkItemDirectlyInput fwidi = new FinishWorkItemDirectlyInput();
                    fwidi.setProcessCode(tempProcessCode);
                    fwidi.setRemark(this.inputInfor.getRemark() == null ? "" : this.inputInfor.getRemark());
                    fwidi.setBsnsId(this.bsnsId);
                    xServiceResult = bpmService.approvalPass(context, fwidi);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    this.inputInfor.setPass(true);
                    break;
                case WorkflowState_Reject:
                    ApprovalRejectInput approvalRejectInput = new ApprovalRejectInput();
                    approvalRejectInput.setProcessCode(tempProcessCode);
                    approvalRejectInput.setBackReason(this.inputInfor.getRemark() == null ? "" : this.inputInfor.getRemark());
                    approvalRejectInput.setBsnsId(this.bsnsId);
                    xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
                    xServiceResult.throwIfFail();
                    this.inputInfor.setSubmitSuccess(true);
                    break;
            }

        } catch (Exception ex) {
            this.inputInfor.setSubmitSuccess(false);
            this.inputInfor.setSubmitResultMessage(ex.toString());
        }

        return this.inputInfor;
    }

    private HashMap<String, String> getUsingInforForSubmit() {
        HashMap<String, String> result = new HashMap<>();
        if (this.auditstatus != null) {
            if (this.inputInfor.getExtendName() == null)
                this.inputInfor.setExtendName("");

            if (this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Submit)
                    || this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Reject)
                    || this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Pass)
            ) {
                result.put("processCode", this.processCode);
                result.put("bsnsType", this.bsnsType);
                result.put("extendName", this.extendName + this.inputInfor.getExtendName());
            }
            {
                result.put("processCode", this.processCodeChange);
                result.put("bsnsType", this.bsnsTypeChange);
                result.put("extendName", this.extendNameChange + this.inputInfor.getExtendName());
            }
        } else {
            result.put("processCode", "");
            result.put("bsnsType", "");
            result.put("extendName", "");
        }
        return result;
    }

    private HashMap<String, String> getUsingInforForButtonList() {
        HashMap<String, String> result = new HashMap<>();
        if (this.auditstatus != null) {
            if (this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Submit)
                    || this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Reject)
            ) {
                result.put("processCode", this.processCode);
                result.put("bsnsType", this.bsnsType);
                result.put("extendName", this.extendName);
            } else if (this.auditstatus.equals(OperationManagementWorkflowBehaviour.WorkflowState_Pass)) {
                result.put("processCode", this.processCodeChange);
                result.put("bsnsType", this.bsnsTypeChange);
                result.put("extendName", this.extendNameChange);
            }
        } else {
            result.put("processCode", this.processCode);
            result.put("bsnsType", this.bsnsType);
            result.put("extendName", this.extendName);
        }
        return result;
    }

    private static boolean checkHasTodo(XContext context, String bsnsId, String bsnsType) {
        if (StringUtils.isBlank(bsnsId) || StringUtils.isBlank(bsnsType))
            return false;
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);
        CheckHasTodoInput chti = new CheckHasTodoInput();
        chti.setBsnsId(bsnsId);
        chti.setBsnsType(bsnsType);
        XSingleResult<CheckHasTodoOutput> result = processService.checkHasTodo(context, chti);
        result.throwIfFail();
        return result.getResult().getHasTodo();
    }

    private static boolean checkInRole(XContext context, String processCode, String userId) {

        boolean result = false;
        SystemUserRoleRelService systemUserRoleRelService = context.getBean(SystemUserRoleRelService.class);
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);

        GetProcessFirstRoleInput chti = new GetProcessFirstRoleInput();
        chti.setProcessCode(processCode);
        XListResult<GetProcessFirstRoleOutput> rrList = processService.getProcessFirstRoles(context, chti);

        GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
        getSysUserRoleRelInput.setUserId(userId);

        List<GetSysUserRoleRelOutput> getSysUserRoleRelOutputs = systemUserRoleRelService.getSysRoleByUserId(context, getSysUserRoleRelInput).getResult();
        for (int i = 0; i < getSysUserRoleRelOutputs.size(); i++) {
            GetSysUserRoleRelOutput getSysUserRoleRelOutput = getSysUserRoleRelOutputs.get(i);
            //获取这个流程的参与者角色id，与 当前用户角色
            result = rrList.getResult().stream().anyMatch(c -> c.getRoleId().equals(getSysUserRoleRelOutput.getRoleId()));
            if (result)
                break;
        }
        return result;
    }

    //获取流程第一个节点的参与者信息（角色信息）与当前登陆人角色比较
    public boolean checkInRoleOrUserId(XContext context, String processCode, String userId) {

        boolean result = false;
        SystemUserRoleRelService systemUserRoleRelService = context.getBean(SystemUserRoleRelService.class);
        BpsWorkFlowProcessService processService = context.getBean(BpsWorkFlowProcessService.class);

        GetProcessFirstRoleInput chti = new GetProcessFirstRoleInput();
        chti.setProcessCode(processCode);
        XListResult<GetProcessFirstRoleOutput> rrList = processService.getProcessFirstRoles(context, chti);

        GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
        getSysUserRoleRelInput.setUserId(userId);

        List<GetSysUserRoleRelOutput> getSysUserRoleRelOutputs = systemUserRoleRelService.getSysRoleByUserId(context, getSysUserRoleRelInput).getResult();
        for (int i = 0; i < getSysUserRoleRelOutputs.size(); i++) {
            GetSysUserRoleRelOutput getSysUserRoleRelOutput = getSysUserRoleRelOutputs.get(i);
            //获取这个流程的参与者角色id，与 当前用户角色
            //result = rrList.getResult().stream().anyMatch(c -> c.getRoleId().equals(getSysUserRoleRelOutput.getRoleId()));
            //result = rrList.getResult().contains(getSysUserRoleRelOutput.getRoleId());
            for (GetProcessFirstRoleOutput output : rrList.getResult()) {
                result = output.getRoleId().equals(getSysUserRoleRelOutput.getRoleId());
                if (result) {
                    return result;
                }
            }
            if (result)
                break;
        }
        return result;
    }

    public boolean checkInRoleOrUserIds(XContext context, String processCode, String userId) {
        boolean result = false;
        SystemUserRoleRelService systemUserRoleRelService = context.getBean(SystemUserRoleRelService.class);
        //查询流程下的所有角色
        BpsWorkFlowProcessService contextBean = context.getBean(BpsWorkFlowProcessService.class);
        QueryProcessRolesInput queryProcessRolesInput = new QueryProcessRolesInput();
        queryProcessRolesInput.setProcessCode(processCode);
        XListResult<QueryProcessRolesOutput> listResult = contextBean.queryProcessRoles(context, queryProcessRolesInput);

        GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
        getSysUserRoleRelInput.setUserId(userId);
        List<GetSysUserRoleRelOutput> getSysUserRoleRelOutputs = systemUserRoleRelService.getSysRoleByUserId(context, getSysUserRoleRelInput).getResult();

        for (int i = 0; i < getSysUserRoleRelOutputs.size(); i++) {
            GetSysUserRoleRelOutput getSysUserRoleRelOutput = getSysUserRoleRelOutputs.get(i);
            for (QueryProcessRolesOutput queryProcessRolesOutput : listResult.getResult()) {
                result = queryProcessRolesOutput.getRoleId().equals(getSysUserRoleRelOutput.getRoleId());
            }
            if (result)
                break;
        }
        return result;
    }


}
