package pps.core.task.job;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import pps.cloud.dispatch.enums.TaskPlanAckImsTypeDic;
import pps.cloud.dispatch.service.DispTaskPlanJobService;
import pps.cloud.dispatch.service.data.TaskPlanImsAckInput;
import pps.cloud.transfer.service.TransferService;
import pps.cloud.transfer.service.data.AddImsLogInput;
import pps.core.common.config.MqConfig;
import pps.core.common.mq.AckImsMqMessage;
import pps.core.task.config.TaskConfig;
import pps.core.task.model.AckImsModel;
import pps.core.task.model.AckImsRespModel;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.mq.XMqConsumeQueue;
import xstartup.base.mq.XMqConsumer;
import xstartup.base.mq.XMqFactory;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@XText("作业计划反馈结果给ims任务")
@XService
public class TaskPlanAckImsJob implements XJob {

    private static Logger logger = LoggerFactory.getLogger(TaskPlanAckImsJob.class);

    @XCronTrigger(value = XCronTrigger.PRE_5S)
    @Override
    public XServiceResult execute(XContext context) {
        try {
            DispTaskPlanJobService taskPlanJobService = context.getBean(DispTaskPlanJobService.class);
            TransferService transferService = context.getBean(TransferService.class);
            RestTemplate restTemplate = new RestTemplate();
            final String ackImsMq = MqConfig.ackImsMq.getValue(context);
            final String ackImsUrl = TaskConfig.ackImsUrl.getValue(context);
            XMqConsumeQueue consumeQueue = new XMqConsumeQueue.Builder()
                    .queue(ackImsMq)
                    .build();
            XMqConsumer<AckImsMqMessage> consumer = XMqFactory.createConsumer(context, consumeQueue, AckImsMqMessage.class);

            XSingleResult<AckImsMqMessage> peekResult = consumer.peek(context, AckImsMqMessage.class);
            while (peekResult.isSuccess() && peekResult.getResult() != null) {
                AckImsMqMessage result = peekResult.getResult();
                AddImsLogInput imsLogInput = new AddImsLogInput();
                try {
                    List list = new ArrayList();
                    AckImsModel ackImsModel = XCopyUtils.copyNewObject(result, AckImsModel.class);
                    list.add(ackImsModel);
                    imsLogInput.setPlanId(result.getPps_id());
                    imsLogInput.setPlanCode(result.getPps_num());
                    imsLogInput.setUrl(ackImsUrl);
                    imsLogInput.setReqParam(XJsonUtils.toJson(list));

                    HttpHeaders headers = new HttpHeaders();
                    headers.add("From", "Y");
                    headers.add("Content-Type", "application/json");
                    context.getLogger().info("发送到IMS的url------>"+ackImsUrl+",参数----->"+ XJsonUtils.toJson(list));
                    // 组装请求体
                    HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity(XJsonUtils.toJson(list), headers);
                    //发起请求
                    Object res = restTemplate.postForObject(ackImsUrl, request, Object.class);
                    context.getLogger().info("接收到IMS的响应------>"+ XJsonUtils.toJson(res));
                    imsLogInput.setRespParam(XJsonUtils.toJson(res));

                    AckImsRespModel ackImsRespModel = XJsonUtils.toObject(XJsonUtils.toJson(res), AckImsRespModel.class);
                    //更新作业计划 ims通知状态
                    if(ackImsRespModel!=null){
                        TaskPlanImsAckInput ackInput = new TaskPlanImsAckInput();
                        ackInput.setTaskPlanId(result.getPps_id());
                        if(ackImsRespModel.getCode()==0){
                            imsLogInput.setStatus(1);
                            ackInput.setStatus(TaskPlanAckImsTypeDic.Finish_Notify.getValue());
                        }else {
                            imsLogInput.setStatus(2);
                            ackInput.setStatus(TaskPlanAckImsTypeDic.Fail_Notify.getValue());
                        }
                        XServiceResult serviceResult = taskPlanJobService.updateImsTaskPlan(context, ackInput);
                        serviceResult.throwIfFail();
                    }
                } catch (Exception e) {
                    context.getLogger().error("消费失败：" + e.getMessage());
                    imsLogInput.setStatus(2);
                    imsLogInput.setRespParam(e.getMessage());
                } finally {
                    consumer.ack(context, peekResult.getResult()); //消费完这个数据后和kafka确认消费
                    //与IMS 交互入库
                    imsLogInput.setCreateTime(new Date());
                    transferService.addImsLog(context,imsLogInput);
                }
                peekResult = consumer.peek(context, AckImsMqMessage.class);
            }
        } catch (Exception e) {
            context.getLogger().error("消费失败：" + e.getMessage());
        }
        return XServiceResult.OK;
    }
}
