package pps.core.task.job;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import pps.cloud.trade.service.TradeDataSyncService;
import pps.cloud.trade.service.data.TradeSyncDataParamInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.CompletableFuture;

/**
 * @ClassName: TradeSyncDataJob
 * @Author : ado
 * @Date: 2023-03-17 9:22
 * @Description:  交易数据同步
 */

@XText("交易数据同步定时任务,同步频率10分钟一次")
@XService
@Slf4j
public class TradeSyncDataFirstJob implements XJob {

    @XCronTrigger(value = "0 0/10 * * * ?")
    @Override
    public XServiceResult execute(XContext xContext) {
        try {
            log.info("定时同步交易数据数据，同步频率10分钟一次");
            TradeDataSyncService dataSyncService = xContext.getBean(TradeDataSyncService.class);
            // 参数
            TradeSyncDataParamInput input = new TradeSyncDataParamInput();
            //分页参数
            if (ObjectUtils.isEmpty(input.getPageNo()) && ObjectUtils.isEmpty(input.getPageSize())) {
                input.setPageNo(1);
                input.setPageSize(500);
            }


            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            //当前日期
            Date currentDate = new Date();
            String strCurrentDate = sdf.format(currentDate);
            //前三天日期
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(sdf.parse(strCurrentDate));
            calendar.add(Calendar.DAY_OF_MONTH, -2);
            Date beforeDate = calendar.getTime();
            String strBeforeDate = sdf.format(beforeDate);

            input.setCreatedAtStart(strBeforeDate + " 00:00:00");
            input.setCreatedAtEnd(strCurrentDate + " 23:59:59");

            //同步 mkt_month_designate_approval
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDesignateApprovalData(xContext, input));

            //同步 mkt_month_designate_detail
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDesignateDetailData(xContext, input));

            //同步 mkt_month_designate_down
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDesignateDownData(xContext, input));


            //同步 mkt_month_designate_up
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDesignateUpData(xContext, input));
            //同步 mkt_month_detail
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDetailData(xContext, input));

            //同步 mkt_month_detail_change
            CompletableFuture.runAsync(() -> dataSyncService.syncMktMonthDetailChangeData(xContext, input));


            //同步 month_plan_transform
            CompletableFuture.runAsync(() -> dataSyncService.syncMonthPlanTransformData(xContext, input));

            //同步 month_plan_transform_config
            CompletableFuture.runAsync(() ->  dataSyncService.syncMonthPlanTransformConfigData(xContext, input));


        } catch (Exception e) {
            log.error("数据同步失败， fail");
            e.printStackTrace();
        }

        return XServiceResult.OK;
    }


}
