package pps.core.task.job;

import lombok.extern.slf4j.Slf4j;
import pps.cloud.trade.service.TradeDataSyncService;
import pps.cloud.trade.service.data.TradeSyncDataParamInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.CompletableFuture;

/**
 * @ClassName: TradeSyncDataJob
 * @Author : ado
 * @Date: 2023-03-17 9:22
 * @Description:  交易数据同步
 */

@XText("交易数据同步定时任务,同步频率，1天一次")
@XService
@Slf4j
public class TradeSyncDataThirdJob implements XJob {

    @XCronTrigger(value = "0 0 0 */1 * ?")
    @Override
    public XServiceResult execute(XContext xContext) {
        try {
            log.info("定时同步数据, 同步频率，1天一次");
            TradeDataSyncService dataSyncService = xContext.getBean(TradeDataSyncService.class);
            TradeSyncDataParamInput input = new TradeSyncDataParamInput();
            input.setPageNo(1);
            input.setPageSize(500);


            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            //当前日期
            Date currentDate = new Date();
            String strCurrentDate = sdf.format(currentDate);
            //前一天日期
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(sdf.parse(strCurrentDate));
            calendar.add(Calendar.DAY_OF_MONTH, -2);
            Date beforeDate = calendar.getTime();
            String strBeforeDate = sdf.format(beforeDate);

            input.setCreatedAtStart(strBeforeDate + " 00:00:00");
            input.setCreatedAtEnd(strCurrentDate + " 23:59:59");

            //同步 kd_company
            CompletableFuture.runAsync(() -> dataSyncService.syncKdCompanyData(xContext, input));

            //同步 pipechina_sku_extra
            CompletableFuture.runAsync(() -> dataSyncService.syncPipechinaSkuExtraData(xContext, input));

            //同步 pipechina_point
            CompletableFuture.runAsync(() -> dataSyncService.syncPipechinaPointData(xContext, input));

            //同步 pipechina_station
            CompletableFuture.runAsync(() -> dataSyncService.syncPipechinaStationData(xContext, input));

            //同步 pipechina_source
            CompletableFuture.runAsync(() -> dataSyncService.syncPipechinaSourceData(xContext, input));

            CompletableFuture.runAsync(() -> dataSyncService.syncPartnerCompanyData(xContext, input));

            CompletableFuture.runAsync(() -> dataSyncService.syncPartnerShipperPointData(xContext, input));


            //全量
            //同步 pipechina_segment
            CompletableFuture.runAsync(() -> dataSyncService.syncPipechinaSegmentData(xContext, input));

            CompletableFuture.runAsync(() -> dataSyncService.syncShipperUnbalanceData(xContext, input));



        } catch (Exception e) {
            log.error("数据同步失败， fail");
            e.printStackTrace();
        }

        return XServiceResult.OK;
    }


}
